<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Helpers;

class ParentRelatedListRetriever
{
    protected $relateFields;

    protected $blackListedRelateFields = [];

    public function __construct()
    {
    }

    public function setRelateParentList(\SugarBean $eventBean, \SugarBean $targetEntityBean, array $linkNames)
    {
        if (empty($eventBean) || empty($targetEntityBean) || empty($linkNames)) {
            return;
        }

        [$relatedData, $partRelatedData] = $this->getAllRelatedData($targetEntityBean, $linkNames);
        $this->createParentRelatedConnections($eventBean, $relatedData);
        $this->createParentRelatedPartConnections($eventBean, $partRelatedData);
    }

    protected function getAllRelatedData(\SugarBean $targetEntityBean, $linkNames)
    {
        $relatedData = [];
        $historyData = [];

        foreach ($linkNames as $key1 => $linkData) {
            $firstLevelLinkName = $linkData['linkLevel1'];
            $firstLevelLinkData = $this->getRelatedDataByParams($targetEntityBean, $firstLevelLinkName);
            $relatedModule = ((!empty($firstLevelLinkData) && !empty($firstLevelLinkData['relatedModule'])) ? $firstLevelLinkData['relatedModule'] : '');
            $linkPathInfo = '';
            if (!empty($linkData['linkLevel1'])) {
                $linkPathInfo .= $linkData['linkLevel1'];
            }
            $linkPathInfo .= ':';
            if (!empty($linkData['linkLevel2'])) {
                $linkPathInfo .= $linkData['linkLevel2'];
            }
            $linkPathInfo .= ':';
            if (!empty($linkData['linkLevel3'])) {
                $linkPathInfo .= $linkData['linkLevel3'];
            }
            $firstLevelRelatedHistory = array(
                'module' => $targetEntityBean->getModuleName(),
                'id' => $targetEntityBean->id,
                'link' => $firstLevelLinkName,
                'related_module' => $relatedModule,
                'link_path_info' => $linkPathInfo,
                'key' => $linkData['key'],
            );

            if (empty($linkData['linkLevel2'])) {
                if (!empty($firstLevelLinkData)) {
                    $firstLevelRelatedHistory['related_ids'] = $firstLevelLinkData['relatedIds'];
                    $firstLevelLinkData['history'] = array($firstLevelRelatedHistory);
                    $firstLevelLinkData['linkLabel'] = $linkData['linkLabel'];
                    $relatedData[] = $firstLevelLinkData;
                }
                continue;
            }

            if (empty($firstLevelLinkData)) {
                $historyData[] = array($firstLevelRelatedHistory);
            }

            $secondLevelTargetModuleName = $firstLevelLinkData['relatedModule'];
            $secondLevelLinkName = $linkData['linkLevel2'];
            foreach ($firstLevelLinkData['relatedIds'] as $secondLevelRelatedId) {
                $firstLevelRelatedHistory['related_id'] = $secondLevelRelatedId;

                $secondLevelEntityBean = \BeanFactory::retrieveBean($secondLevelTargetModuleName, $secondLevelRelatedId, array('disable_row_level_security' => true));
                if (empty($secondLevelEntityBean)) {
                    continue;
                }
                $secondLevelLinkData = $this->getRelatedDataByParams($secondLevelEntityBean, $secondLevelLinkName);
                $secondLevelRelatedHistory = array(
                    'module' => $secondLevelEntityBean->getModuleName(),
                    'id' => $secondLevelEntityBean->id,
                    'link' => $secondLevelLinkName,
                    'related_module' => $secondLevelLinkData['relatedModule'],
                    'link_path_info' => $linkData['linkLevel1'] . ':' . $linkData['linkLevel2'] . ':' . $linkData['linkLevel3'],
                    'key' => $linkData['key'],
                );

                if (empty($linkData['linkLevel3'])) {
                    if (!empty($secondLevelLinkData)) {
                        $secondLevelRelatedHistory['related_ids'] = $secondLevelLinkData['relatedIds'];
                        $secondLevelLinkData['history'] = array($firstLevelRelatedHistory, $secondLevelRelatedHistory);
                        $secondLevelLinkData['linkLabel'] = $linkData['linkLabel'];
                        $relatedData[] = $secondLevelLinkData;
                    }
                    continue;
                }

                if (empty($secondLevelLinkData)) {
                    $historyData[] = array($firstLevelRelatedHistory, $secondLevelRelatedHistory);
                }

                $thirdLevelTargetModuleName = $secondLevelLinkData['relatedModule'];
                $thirdLevelLinkName = $linkData['linkLevel3'];
                foreach ($secondLevelLinkData['relatedIds'] as $thirdLevelRelatedId) {
                    $secondLevelRelatedHistory['related_id'] = $thirdLevelRelatedId;

                    $thirdLevelEntityBean = \BeanFactory::retrieveBean($thirdLevelTargetModuleName, $thirdLevelRelatedId, array('disable_row_level_security' => true));
                    if (empty($thirdLevelEntityBean)) {
                        continue;
                    }
                    $thirdLevelLinkData = $this->getRelatedDataByParams($thirdLevelEntityBean, $thirdLevelLinkName);
                    $thirdLevelRelatedHistory = array(
                        'module' => $thirdLevelEntityBean->getModuleName(),
                        'id' => $thirdLevelEntityBean->id,
                        'link' => $thirdLevelLinkName,
                        'related_module' => $thirdLevelLinkData['relatedModule'],
                        'link_path_info' => $linkData['linkLevel1'] . ':' . $linkData['linkLevel2'] . ':' . $linkData['linkLevel3'],
                        'key' => $linkData['key'],
                    );

                    if (!empty($thirdLevelLinkData)) {
                        $thirdLevelRelatedHistory['related_ids'] = $thirdLevelLinkData['relatedIds'];
                        $thirdLevelLinkData['history'] = array($firstLevelRelatedHistory, $secondLevelRelatedHistory, $thirdLevelRelatedHistory);
                        $thirdLevelLinkData['linkLabel'] = $linkData['linkLabel'];
                        $relatedData[] = $thirdLevelLinkData;
                    } else {
                        $historyData[] = array($firstLevelRelatedHistory, $secondLevelRelatedHistory, $thirdLevelRelatedHistory);
                    }
                }
            }
        }
        return array($relatedData, $historyData);
    }

    protected function getRelatedDataByParams(\SugarBean $entityBean, $linkName)
    {
        $relatedData = array();
        if (empty($linkName)) {
            return array();
        }

        if (!$entityBean->load_relationship($linkName)) {
            return array();
        }

        /** @var \Link2 $link */
        $link = $entityBean->getFieldValue($linkName);
        if (!empty($link)) {
            $relatedModuleName = $link->getRelatedModuleName();

            if ($link->isParentRelationship()) {
                if (!empty($entityBean->parent_type) && !empty($entityBean->parent_id) && $entityBean->parent_type == $link->getRelatedModuleName()) {
                    $relatedIds = array($entityBean->parent_id);
                }
            } else {
                $relatedIds = $link->get();
            }

            if (!empty($relatedIds)) {
                $relatedData = array(
                    'relatedModule' => $relatedModuleName,
                    'relatedIds' => $relatedIds,
                    'linkName' => $linkName,
                );
            }
        }
        return $relatedData;
    }

    protected function createParentRelatedConnections(\SugarBean $eventBean, array $relatedData)
    {
        foreach ($relatedData as $relatedItem) {
            $parentType = $relatedItem['relatedModule'];
            $linkName = $relatedItem['linkName'];
            foreach ($relatedItem['relatedIds'] as $parentId) {
                $ntdvRelates = \BeanFactory::getBean('PX_NeedToDoRelates');
                $ntdvRelates->id = \Sugarcrm\Sugarcrm\Util\Uuid::uuid1();
                $ntdvRelates->new_with_id = true;
                $ntdvRelates->parent_type = $parentType;
                $ntdvRelates->parent_id = $parentId;
                $ntdvRelates->relate_type = 'Target';
                $ntdvRelates->event_id = $eventBean->id;
                $ntdvRelates->link_name = $linkName;
                $ntdvRelates->link_label = $relatedItem['linkLabel'];
                $ntdvRelates->save();

                $this->createParentRelatedHistory($relatedItem['history'], $eventBean, $ntdvRelates);
            }
        }
    }

    protected function createParentRelatedPartConnections(\SugarBean $eventBean, array $partRelatedData)
    {
        foreach ($partRelatedData as $partRelatedItem) {
            $this->createParentRelatedHistory($partRelatedItem, $eventBean);
        }
    }

    protected function createParentRelatedHistory($historyData, \SugarBean $eventBean, \SugarBean $ntdvRelateBean = null)
    {
        $db = \DBManagerFactory::getInstance();
        $ntdvRelateId = !empty($ntdvRelateBean) ? $ntdvRelateBean->id : '';
        foreach ($historyData as $history) {
            $relatedId = '';
            if (!empty($history['related_id'])) {
                $relatedId = $history['related_id'];
            } elseif (!empty($history['related_ids']) && !empty($ntdvRelateBean) && in_array($ntdvRelateBean->parent_id, $history['related_ids'])) {
                $relatedId = $ntdvRelateBean->parent_id;
            }

            if (empty($relatedId) && !empty($ntdvRelateBean)) {
                continue;
            }

            $ntdvRelatesTreeId = \Sugarcrm\Sugarcrm\Util\Uuid::uuid1();
            $insertSQL = "
                INSERT INTO px_ntdv_relates_tree
                    (id, event_id, ntdv_relate_id, lhs_module, lhs_id, relate_link_name, rhs_module, rhs_id, link_path, chain_key)
                VALUES
                (
                    '$ntdvRelatesTreeId',
                    '{$eventBean->id}',
                    '$ntdvRelateId',
                    '" . $history['module'] . "',
                    '" . $history['id'] . "',
                    '" . $history['link'] . "',
                    '" . $history['related_module'] . "',
                    '$relatedId',
                    '" . $history['link_path_info'] . "',
                    '" . $history['key'] . "'                    
                )
            ";
            $db->query($insertSQL);
        }
    }

    public function deleteOldLinks($eventId)
    {
        $db = \DBManagerFactory::getInstance();
        $cleanUpSQL = "DELETE FROM px_ntdv_parents WHERE event_id='$eventId'";
        $db->query($cleanUpSQL);
    }

    public function deleteOldLinksTree($eventId)
    {
        $db = \DBManagerFactory::getInstance();
        $cleanUpSQL = "DELETE FROM px_ntdv_relates_tree WHERE event_id='$eventId'";
        $db->query($cleanUpSQL);
    }

    public function deleteQueuedRelationsOldLinksTree($eventId)
    {
        $db = \DBManagerFactory::getInstance();
        $cleanUpSQL = "DELETE FROM px_ntdv_refresh_relations_queue WHERE event_id='$eventId'";
        $db->query($cleanUpSQL);
    }


    protected function fieldHasNameAttribute($fieldDefs)
    {
        return !empty($fieldDefs['name']);
    }

    protected function fieldIsRelated($fieldDefs)
    {
        return !empty($fieldDefs['type']) && $fieldDefs['type'] == 'link';
    }

    protected function relateFieldIsValid($targetBean, $fieldDefs)
    {
        $relationship = $targetBean->load_relationship($fieldDefs['name']);
        return !empty($relationship);
    }

    protected function getFieldLabel($targetBean, $fieldDefs)
    {
        $targetModule = $targetBean->getModuleName();
        $fieldName = $fieldDefs['name'];
        $labelKey = !empty($fieldDefs['vname']) ? $fieldDefs['vname'] : '';
        $labelValue = translate($labelKey, $targetModule);
        $relatedModuleLabel = $this->getRelatedModuleLabel($targetBean, $fieldName);

        if (empty($labelValue)) {
            $labelValue = $fieldName;
        }
        $labelValue = $this->removeColonFromTheEndOfSomeLabels($labelValue);
        return $this->translateFieldLabel($relatedModuleLabel, $fieldName, $labelValue);
    }

    protected function translateFieldLabel($moduleLabel, $linkName, $linkLabel)
    {
        $moduleLabelLower = mb_strtolower($moduleLabel);
        $linkNameLower = mb_strtolower($linkName);
        $linkLabelLower = mb_strtolower($linkLabel);

        $translateString = 'LBL_ENTITY_PARENT_ITEM_TPL_MODULE';
        if ($linkLabelLower != $moduleLabelLower && $linkLabelLower != $linkNameLower) {
            $translateString = 'LBL_ENTITY_PARENT_ITEM_TPL_LINK_AND_MODULE';
        } else if($linkLabelLower != $moduleLabelLower) {
            $translateString = 'LBL_ENTITY_PARENT_ITEM_TPL_MODULE';
        } else if($linkLabelLower != $linkNameLower) {
            $translateString = 'LBL_ENTITY_PARENT_ITEM_TPL_LINK';
        }

        $tpl = translate($translateString, 'PX_NeedToDoEventTypes');
        return str_replace(array('%LINK_LABEL%', '%LINK_NAME%', '%RELATED_MODULE_NAME%'), array($linkLabel, $linkName, $moduleLabel), $tpl);
    }

    protected function getRelatedModuleLabel($targetBean, $linkName)
    {
        $link = $targetBean->getFieldValue($linkName);

        if (empty($link) || !method_exists($link, 'getRelatedModuleName') || !method_exists($link, 'getType')) {
            return '';
        }

        $linkType = $link->getType();
        $relatedModuleName = $link->getRelatedModuleName();

        if ($linkType == 'one') {
            $relatedModuleLabel = $GLOBALS['app_list_strings']['moduleListSingular'][$relatedModuleName];
        } elseif ($linkType == 'many') {
            $relatedModuleLabel = $GLOBALS['app_list_strings']['moduleList'][$relatedModuleName];
        }

        if (empty($relatedModuleLabel)) {
            $relatedModuleLabel = $relatedModuleName;
        }

        return $relatedModuleLabel;
    }

    private function removeColonFromTheEndOfSomeLabels($label)
    {
        return preg_replace ( '/(.*)(:+)$/' , '$1' , trim($label));
    }

}