<?php

class PxNtdvGlobalLogicHooks
{
    public const INDEX_STATUS_IN_PROCESS = 'In Process';
    protected static $targetEventTypes;
    protected static $relatedData = [];

    public function needToDoEventModulesIsInstalled()
    {
        $needToDoEventTypeBean = \BeanFactory::getBean('PX_NeedToDoEventTypes');
        return !empty($needToDoEventTypeBean);
    }

    public function firstBeforeSave(\SugarBean $bean, $event, $args)
    {
        if ($this->isSugarBpmModule($bean)) {
            return;
        }
        if (!$this->needToDoEventModulesIsInstalled()) {
            return;
        }

        $this->collectEventTypes();
    }

    public function firstBeforeRelationshipAdd($bean, $event, $arguments)
    {
        if ($this->isSugarBpmModule($bean)) {
            return;
        }
        if (!$this->needToDoEventModulesIsInstalled()) {
            return;
        }

        $this->collectEventTypes();
    }

    public function firstBeforeRelationshipDelete($bean, $event, $arguments)
    {
        if ($this->isSugarBpmModule($bean)) {
            return;
        }
        if (!$this->needToDoEventModulesIsInstalled()) {
            return;
        }

        $this->collectEventTypes();
    }

    protected function collectEventTypes()
    {
        if (isset(self::$targetEventTypes) || !$this->needToDoEventModulesIsInstalled()) {
            return;
        }

        $needToDoEventTypes = \BeanFactory::getBean('PX_NeedToDoEventTypes');

        $sugarQuery = new \SugarQuery();
        $sugarQuery->from($needToDoEventTypes);
        $sugarQuery->select(array('id', 'target_module', 'target_attribute_name', 'target_entity_filter', 'target_entity_parent_list'));
        $sugarQuery->where()->equals('status', \PX_NeedToDoEventTypes::ENABLED_STATUS);
        $queryResult = $sugarQuery->execute();
        foreach ($queryResult as $row) {
            $row['target_entity_parent_list'] = $needToDoEventTypes->getParentRelatedNames($row['target_entity_parent_list']);
            if (!empty($row['target_entity_parent_list'])) {
                $targetModule1 = $row['target_module'];
                $relatedBean1 = \BeanFactory::getBean($targetModule1);
                foreach ($row['target_entity_parent_list'] as $parentItem) {
                    $linkInfo1 = array();
                    $linkInfo2 = array();
                    $linkInfo3 = array();

                    //first level link
                    $linkInfo1 = $this->getLinkInfo($relatedBean1, $parentItem['linkLevel1']);
                    $linkInfo1['eventTypeId'] = $row['id'];
                    $linkInfo1['level'] = 1;
                    $linkInfo1['target_entity_parent_list'] = $row['target_entity_parent_list'];
                    $linkInfo1['key'] = $parentItem['key'];
                    self::$relatedData[] = $linkInfo1;

                    //second level link
                    if (!empty($parentItem['linkLevel2'])) {
                        $relatedBean2 = \BeanFactory::getBean($linkInfo1['relatedModule']);
                        if(empty($relatedBean2)) {
                            continue;
                        }
                        $linkInfo2 = $this->getLinkInfo($relatedBean2, $parentItem['linkLevel2']);
                        $linkInfo2['eventTypeId'] = $row['id'];
                        $linkInfo2['level'] = 2;
                        $linkInfo2['history'] = array($linkInfo1);
                        $linkInfo2['target_entity_parent_list'] = $row['target_entity_parent_list'];
                        $linkInfo2['key'] = $parentItem['key'];
                        self::$relatedData[] = $linkInfo2;
                    }

                    //third level link
                    if (!empty($parentItem['linkLevel3']) && !empty($linkInfo2)) {
                        $relatedBean3 = \BeanFactory::getBean($linkInfo2['relatedModule']);
                        if (empty($relatedBean3)) {
                            continue;
                        }
                        $linkInfo3 = $this->getLinkInfo($relatedBean3, $parentItem['linkLevel3']);
                        $linkInfo3['eventTypeId'] = $row['id'];
                        $linkInfo3['level'] = 3;
                        $linkInfo3['history'] = array($linkInfo1, $linkInfo2);
                        $linkInfo3['target_entity_parent_list'] = $row['target_entity_parent_list'];
                        $linkInfo3['key'] = $parentItem['key'];
                        self::$relatedData[] = $linkInfo3;
                    }

                }
            }
            self::$targetEventTypes[$row['target_module']][] = $row;
        }
    }

    protected function getLinkInfo(SugarBean $targetBean, $linkName)
    {
        if (!$targetBean->load_relationship($linkName)) {
            return array();
        }

        $link = $targetBean->getFieldValue($linkName);
        $relatedModuleName = $link->getRelatedModuleName();
        $relationshipName = $targetBean->field_defs[$linkName]['relationship'];
        return array(
            'targetModule' => $targetBean->getModuleName(),
            'targetLink' => $linkName,
            'relatedModule' => $relatedModuleName,
            'relationship' => $relationshipName
        );
    }

    private function isSugarBpmModule(\SugarBean $bean): bool
    {
        return str_starts_with($bean->module_name, 'pmse_');
    }

    public function lastAfterSave(\SugarBean $bean, $event, $args)
    {
        if ($this->isSugarBpmModule($bean)) {
            return;
        }
        if (!$this->needToDoEventModulesIsInstalled()) {
            return;
        }

        $this->checkEntityForRelationChanges($bean, $event, $args);


        if (empty(self::$targetEventTypes)) {
            return;
        }

        $targetModule = $bean->getModuleName();
        if (empty(self::$targetEventTypes[$targetModule])) {
            return;
        }

        $this->pushRecordToIndexing($bean, $args, self::$targetEventTypes[$targetModule]);
    }

    protected function checkEntityForRelationChanges(\SugarBean $bean, $event, $args)
    {
        if (!empty($args['dataChanges'])) {
            $relationsChanged = [];
            $relationships = SugarRelationshipFactory::getInstance()->getRelationshipDefs();
            foreach ($args['dataChanges'] as $changedField) {
                if (
                    $changedField['field_name'] == 'parent_id' &&
                    !empty($bean->field_defs['parent_id']['group']) &&
                    $bean->field_defs['parent_id']['group'] == 'parent_name'
                ) {
                    $currentModuleName = $bean->getModuleName();
                    $parentModuleName = $bean->parent_type;

                    foreach ($bean->field_defs as $fieldDef) {
                        if ($fieldDef['type'] == 'link' && !empty($fieldDef['relationship'])) {
                            $linkName = $fieldDef['name'];
                            $relationshipName = $fieldDef['relationship'];
                            if (!empty($relationships[$relationshipName])) {
                                $rel = $relationships[$relationshipName];
                                if (
                                    $rel['relationship_role_column'] == 'parent_type' &&
                                    (
                                        ($rel['rhs_key'] == 'parent_id' && $rel['rhs_module'] == $currentModuleName && $rel['lhs_module'] == $parentModuleName) ||
                                        ($rel['lhs_key'] == 'parent_id' && $rel['lhs_module'] == $currentModuleName && $rel['rhs_module'] == $parentModuleName)
                                    )
                                ) {
                                    $relationsChanged[$fieldDef['link']] = array(
                                        'module' => $bean->getModuleName(),
                                        'id' => $bean->id,
                                        'related_module' => $parentModuleName,
                                        'before_related_id' => $changedField['before'],
                                        'after_related_id' => $changedField['after'],
                                        'link' => $linkName,
                                        'relationship' => $relationshipName,
                                    );
                                }
                            }
                        }
                    }
                } elseif ($changedField['data_type'] == 'id') {
                    $changedFieldName = $changedField['field_name'];
                    foreach ($bean->field_defs as $fieldDef) {
                        if (
                            isset($fieldDef['link']) &&
                            !isset($relationsChanged[$fieldDef['link']]) &&
                            $fieldDef['type'] == 'relate' &&
                            $fieldDef['id_name'] == $changedFieldName &&
                            !empty($fieldDef['link'])
                        ) {
                            $linkDefs = $this->getLinkDefs($bean, $fieldDef['link']);
                            if (!$bean->load_relationship($fieldDef['link'])) {
                                continue;
                            }
                            /** @var Link2 $link */
                            $link = $bean->getFieldValue($fieldDef['link']);
                            $relationsChanged[$fieldDef['link']] = array(
                                'module' => $bean->getModuleName(),
                                'id' => $bean->id,
                                'related_module' => $link->getRelatedModuleName(),
                                'before_related_id' => $changedField['before'],
                                'after_related_id' => $changedField['after'],
                                'link' => $fieldDef['link'],
                                'relationship' => $linkDefs['relationship'],
                            );
                        }
                    }
                }
            }
        }

        if (!empty($relationsChanged)) {
            foreach ($relationsChanged as $relationInfo) {
                if (!empty($relationInfo['before_related_id'])) {
                    $afterRelationshipDeleteArgs = $relationInfo;
                    $afterRelationshipDeleteArgs['related_id'] = $relationInfo['before_related_id'];
                    $this->firstAfterRelationshipDelete($bean, $event, $afterRelationshipDeleteArgs);
                }

                if (!empty($relationInfo['after_related_id'])) {
                    $afterRelationshipDeleteArgs = $relationInfo;
                    $afterRelationshipDeleteArgs['related_id'] = $relationInfo['after_related_id'];
                    $this->firstAfterRelationshipAdd($bean, $event, $afterRelationshipDeleteArgs);
                }
            }
        }
    }

    protected function getLinkDefs($bean, $linkName)
    {
        $def = [];
        if (empty($bean->field_defs) || empty($bean->field_defs[$linkName]) || empty($bean->field_defs[$linkName]['relationship'])) {
            if (empty($linkDef)) {
                $def = VardefManager::getLinkFieldForRelationship($bean->module_dir, $bean->object_name, $linkName);
            } else {
                $def = $linkDef;
            }
            if (is_array($def) && !isset($def['name'])) {
                if (isset($def[0]['side']) && $def[0]['side'] == 'left') {
                    $def = $def[0];
                } else if (isset($def[1]['side']) && $def[1]['side'] == 'left') {
                    $def = $def[1];
                } else {
                    $def = $def[0];
                }
            }
            if (empty($def['name'])) {
                return [];
            }
        } else {
            $def = $bean->field_defs[$linkName];
        }
        return $def;
    }

    protected function pushRecordToIndexing(\SugarBean $bean, $args, $needToDoEventTypes)
    {
        global $timedate, $current_user;
        $db = \DBManagerFactory::getInstance();
        $currentDatetime = $timedate->nowDb();
        $currentUserId = $current_user->id;
        $needToDoEventTypeBean = \BeanFactory::getBean('PX_NeedToDoEventTypes');
        $needToDoEventBean = \BeanFactory::getBean('PX_NeedToDoEvents');

        foreach ($needToDoEventTypes as $needToDoEventTypeData) {
            $indexStackId = \Sugarcrm\Sugarcrm\Util\Uuid::uuid1();
            $targetModule = $bean->getModuleName();
            $targetRecordId = $bean->id;
            $targetAttrName = $needToDoEventTypeData['target_attribute_name'];
            $targetControlDate = $bean->getFieldValue($targetAttrName); //$args['dataChanges'][$targetAttrName]['after'];
            $needToDoEventTypeId = $needToDoEventTypeData['id'];

            $targetControlDateType = $needToDoEventTypeBean->getTargetAttributeType($targetModule, $targetAttrName);
            $targetControlDate = $needToDoEventBean->getControlDateForDateFormat($targetControlDate, $targetControlDateType);

            $stackIdSQL = "
                SELECT id 
                FROM px_ntdv_indexing_stack 
                WHERE 
                    target_module='$targetModule' AND 
                    target_record_id='$targetRecordId' AND 
                    need_to_do_event_type_id='$needToDoEventTypeId' AND
                    (status IS NULL OR status = '' OR status <> '" . self::INDEX_STATUS_IN_PROCESS . "')
            ";
            $row = $db->fetchOne($stackIdSQL);
            if (!empty($row['id'])) {
                $updateSQL = "
                    UPDATE px_ntdv_indexing_stack
                    SET 
                        target_control_date=" . (empty($targetControlDate) ? 'NULL' : ("'" . $targetControlDate . "'")) . ",
                        date_entered='$currentDatetime',
                        created_by='$currentUserId'
                    WHERE 
                        id='" . $row['id'] . "'
                ";
                $db->query($updateSQL);
            } else {
                $insertSQL = "
                    INSERT INTO px_ntdv_indexing_stack 
                      (id, target_module, target_record_id, need_to_do_event_type_id, target_control_date, date_entered, created_by, description)
                    VALUES 
                      (
                        '" . $indexStackId . "',
                        '" . $targetModule . "',
                        '" . $targetRecordId . "',
                        '" . $needToDoEventTypeId . "',
                        " . (empty($targetControlDate) ? 'NULL' : ("'" . $targetControlDate . "'")) . ",
                        '" . $currentDatetime . "',
                        '" . $currentUserId . "',
                        ''
                      )
                ";
                $db->query($insertSQL);
            }
        }
    }

    public function firstAfterRelationshipAdd($bean, $event, $arguments)
    {
        if ($this->isSugarBpmModule($bean)) {
            return;
        }
        if (!$this->needToDoEventModulesIsInstalled()) {
            return;
        }
        $indexRelations = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Indexing\IndexRelations();

        foreach (self::$relatedData as $item) {
            if ($arguments['relationship'] != $item['relationship']) {
                continue;
            }

            if (
                (
                    $arguments['module'] == $item['relatedModule'] &&
                    $arguments['related_module'] == $item['targetModule']
                ) || (
                    $arguments['related_module'] == $item['relatedModule'] &&
                    $arguments['module'] == $item['targetModule']
                )
            ) {
                if ($arguments['module'] == $item['relatedModule']) {
                    $relatedId = $arguments['id'];
                    $recordId = $arguments['related_id'];
                } elseif ($arguments['related_module'] == $item['relatedModule']) {
                    $relatedId = $arguments['related_id'];
                    $recordId = $arguments['id'];
                }

                $item['module_name'] = $item['targetModule'];
                $item['record_id'] = $recordId;
                $item['related_record_id'] = $relatedId;
                $item['event_type_id'] = $item['eventTypeId'];
                $item['related_module_name'] = $item['relatedModule'];
                $item['link_name'] = $item['targetLink'];

                $indexRelations->addRelation($item);
            }
        }
    }

    public function firstAfterRelationshipDelete($bean, $event, $arguments)
    {
        if ($this->isSugarBpmModule($bean)) {
            return;
        }
        if (!$this->needToDoEventModulesIsInstalled()) {
            return;
        }

        $indexRelations = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Indexing\IndexRelations();

        foreach (self::$relatedData as $item) {
            if ($arguments['relationship'] != $item['relationship']) {
                continue;
            }

            if (
                (
                    $arguments['module'] == $item['relatedModule'] &&
                    $arguments['related_module'] == $item['targetModule']
                ) || (
                    $arguments['related_module'] == $item['relatedModule'] &&
                    $arguments['module'] == $item['targetModule']
                )
            ) {
                if($arguments['module'] == $item['relatedModule']) {
                    $relatedId = $arguments['id'];
                    $recordId = $arguments['related_id'];
                } elseif ($arguments['related_module'] == $item['relatedModule']) {
                    $relatedId = $arguments['related_id'];
                    $recordId = $arguments['id'];
                }

                $indexRelations->removeRelation(
                    array(
                        'module_name' => $item['targetModule'],
                        'record_id' => $recordId,
                        'event_type_id' => $item['eventTypeId'],
                        'related_module_name' => $item['relatedModule'],
                        'related_record_id' => $relatedId,
                        'link_name' => $item['targetLink'],
                    )
                );
            }
        }
    }

}