<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Indexing;

class IndexScheduler
{

    public const INDEX_STATUS_IN_PROCESS = 'In Process';
    public const INDEX_PER_ITERATION_LIMIT = 1000;
    public const REFRESH_RELATIONS_PER_ITERATION_LIMIT = 500;
    protected $db;

    public function __construct()
    {
        $this->db = \DBManagerFactory::getInstance();
    }

    public function index()
    {
        $this->markDataForIndexingToInProcess();
        $dataForIndex = $this->getDataForIndex();

        foreach ($dataForIndex as $data) {
            $this->indexRow($data);
        }
        $this->deleteProcessedData();

        $this->refreshEventRelationships();
    }

    protected function markDataForIndexingToInProcess()
    {
        $sql = "
            UPDATE px_ntdv_indexing_stack 
            SET status='" . self::INDEX_STATUS_IN_PROCESS . "' 
            WHERE id in (
                SELECT id FROM (
                  SELECT id 
                  FROM px_ntdv_indexing_stack
                  WHERE
                        status IS NULL OR
                        status = '' OR 
                        status <> 'In Process' 
                  ORDER BY date_entered ASC
                  LIMIT " . self::INDEX_PER_ITERATION_LIMIT . "
                ) tmp
            )";
        $this->db->query($sql);
    }

    protected function getDataForIndex()
    {
        $dataForIndex = [];
        $sql = "
            SELECT 
                px_ntdv_indexing_stack.target_module target_module, 
                px_ntdv_indexing_stack.target_record_id target_record_id, 
                px_ntdv_indexing_stack.need_to_do_event_type_id need_to_do_event_type_id, 
                px_ntdv_indexing_stack.target_control_date target_control_date, 
                px_ntdv_indexing_stack.date_entered date_entered,
                px_needtodoeventtypes.target_attribute_name target_attribute_name, 
                px_needtodoeventtypes.target_entity_filter target_entity_filter,
                px_needtodoeventtypes.target_attribute_type target_control_date_type
            FROM px_ntdv_indexing_stack
            INNER JOIN px_needtodoeventtypes ON (
              px_ntdv_indexing_stack.need_to_do_event_type_id = px_needtodoeventtypes.id AND px_needtodoeventtypes.deleted = 0 
            )
            WHERE 
                px_ntdv_indexing_stack.status='" . self::INDEX_STATUS_IN_PROCESS . "'
            ORDER BY px_ntdv_indexing_stack.date_entered ASC
            ";
        $rs = $this->db->query($sql);
        $processedData = array();
        while ($row = $this->db->fetchByAssoc($rs)) {
            $uniqueKey = $row['need_to_do_event_type_id'] . $row['target_module'] . $row['target_record_id'];
            if (empty($processedData[$uniqueKey])) {
                $dataForIndex[] = $row;
            } else {
                $processedData[$uniqueKey] = true;
            }
        }
        return $dataForIndex;
    }


    public function indexRow(array $data)
    {
        if (!$this->isTargetDataValid($data)) {
            return;
        }
        $this->indexTargetRecord($data);
    }

    private function isTargetDataValid(array $data)
    {
        return !empty($data['target_module']) && !empty($data['target_record_id']) && !empty($data['need_to_do_event_type_id']);
    }

    private function indexTargetRecord($targetRecord)
    {
        $needToDoEventId = $this->getNeedToDoEventIdByAttributes($targetRecord);

        if ($this->recordFitsTheFilters($targetRecord)) {
            $needToDoEventBean = \BeanFactory::getBean('PX_NeedToDoEvents', $needToDoEventId);
            $needToDoEventBean->control_date = $targetRecord['target_control_date'];
            $needToDoEventBean->control_date_type = $targetRecord['target_control_date_type'];
            $needToDoEventBean->object_type = $targetRecord['target_module'];
            $needToDoEventBean->object_id = $targetRecord['target_record_id'];
            $needToDoEventBean->event_type_id = $targetRecord['need_to_do_event_type_id'];
            $needToDoEventBean->save();
        } else if(!empty($needToDoEventId)) {
            $needToDoEventBean = \BeanFactory::getBean('PX_NeedToDoEvents');
            $this->removeOldNeedToDoEvents($needToDoEventBean, array($needToDoEventId));
        }
    }

    private function recordFitsTheFilters($targetRecord)
    {
        $targetModule = $targetRecord['target_module'];
        $targetRecordID = $targetRecord['target_record_id'];
        $filterDefs = $targetRecord['target_entity_filter'];
        $targetAttributeName = $targetRecord['target_attribute_name'];
        $targetControlDate = $targetRecord['target_control_date'];

        if (empty($targetControlDate)) {
            return false;
        } else if(empty($filterDefs)) {
            return true;
        }

        $filterDefs = json_decode($filterDefs, true);

        $beanFrom = \BeanFactory::getBean($targetModule);
        if (empty($beanFrom) || empty($beanFrom->field_defs[$targetAttributeName])) {
            return false;
        }

        $query = new \SugarQuery();
        $query->from($beanFrom);
        $query->where()->equals('id', $targetRecordID)->isNotEmpty($targetAttributeName);

        require_once 'modules/PX_NeedToDoEventTypes/clients/base/api/PX_NeedToDoEventTypesFilterApi.php';
        $needToDoEventTypesFilterApi = new \PX_NeedToDoEventTypesFilterApi();
        $needToDoEventTypesFilterApi->addTargetFilters($filterDefs, $query->where(), $query);

        $result = $query->execute();
        return !empty($result);
    }

    private function getNeedToDoEventIdByAttributes(array $targetRecord)
    {
        $sugarQuery = new \SugarQuery();
        $eventBean = \BeanFactory::getBean('PX_NeedToDoEvents');
        $sugarQuery->from($eventBean);
        $sugarQuery->select(array('id'));
        $sugarQuery->where()->equals('object_type', $targetRecord['target_module']);
        $sugarQuery->where()->equals('object_id', $targetRecord['target_record_id']);
        $sugarQuery->where()->equals('event_type_id', $targetRecord['need_to_do_event_type_id']);
        $queryResult = $sugarQuery->execute();

        $needToDoEventId = null;
        if (!empty($queryResult) && count($queryResult) > 0) {
            $needToDoEventId = $queryResult[0]['id'];

            $idsToRemove = array();
            foreach ($queryResult as $key => $row) {
                if (!$key) {
                    continue;
                }
                $idsToRemove[] = $row['id'];
            }
            $this->removeOldNeedToDoEvents($eventBean, $idsToRemove);
        }

        return $needToDoEventId;
    }

    private function removeOldNeedToDoEvents(\SugarBean $eventBean, array $idsToRemove)
    {
        if (empty($idsToRemove)) {
            return;
        }

        foreach ($idsToRemove as $idToRemove) {
            $eventBean->mark_deleted($idToRemove);
        }
    }

    protected function deleteProcessedData()
    {
        $sql = "DELETE FROM px_ntdv_indexing_stack WHERE status='" . self::INDEX_STATUS_IN_PROCESS . "'";
        $this->db->query($sql);
    }

    protected function refreshEventRelationships()
    {
        $this->markRelationDataForRefreshToInProcess();
        $dataForIndex = $this->getRelationDataForProcess();

        foreach ($dataForIndex as $data) {
            $this->processRelation($data);
        }
        $this->deleteProcessedRelationData();
    }

    protected function markRelationDataForRefreshToInProcess()
    {
        $sql = "
            UPDATE px_ntdv_refresh_relations_queue 
            SET status='" . self::INDEX_STATUS_IN_PROCESS . "' 
            WHERE id in (
                SELECT id FROM (
                  SELECT id 
                  FROM px_ntdv_refresh_relations_queue
                  WHERE
                        status IS NULL OR
                        status = '' OR 
                        status <> '" . self::INDEX_STATUS_IN_PROCESS . "' 
                  ORDER BY date_entered ASC
                  LIMIT " . self::REFRESH_RELATIONS_PER_ITERATION_LIMIT . "
                ) tmp
            )";
        $this->db->query($sql);
    }

    protected function getRelationDataForProcess()
    {
        $dataForProcess = array();
        $sql = "
            SELECT 
                distinct event_id as event_id
            FROM px_ntdv_refresh_relations_queue
            WHERE 
                status='" . self::INDEX_STATUS_IN_PROCESS . "'
            ORDER BY date_entered ASC
            ";
        $rs = $this->db->query($sql);
        while ($row = $this->db->fetchByAssoc($rs)) {
            $dataForProcess[] = $row['event_id'];
        }
        return $dataForProcess;
    }

    private function processRelation($needToDoEventId)
    {
        /** @var \PX_NeedToDoEvents $needToDoEventBean */
        $needToDoEventBean = \BeanFactory::retrieveBean('PX_NeedToDoEvents', $needToDoEventId);
        if (!empty($needToDoEventBean)) {
            $needToDoEventBean->resetParentList();
        }
    }

    protected function deleteProcessedRelationData()
    {
        $sql = "DELETE FROM px_ntdv_refresh_relations_queue WHERE status='" . self::INDEX_STATUS_IN_PROCESS . "'";
        $this->db->query($sql);
    }

}