<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoEvents\helpers;

class NotifyUsersList{

    protected $needToDoEventType;
    protected $needToDoEvent;
    public const NOTIFY_ASSIGNED_USER_ID = 'notify_assigned_user_id';
    public const NOTIFICATION_ON_STATUS = '0';
    public const NOTIFICATION_COMPLETED_STATUS = '1';
    public const NOTIFICATION_OFF_STATUS = '2';

    public function __construct(\PX_NeedToDoEventTypes $needToDoEventType, \PX_NeedToDoEvents $needToDoEvent)
    {
        $this->needToDoEventType = $needToDoEventType;
        $this->needToDoEvent = $needToDoEvent;
    }

    public function setUsersForNotification(){
        if(!$this->isNotificationAvailable()){
//            $GLOBALS['log']->fatal('1');
            return;
        }

        $this->createUserNotificationList();
    }

    protected function isNotificationAvailable(){
        $exp1 = $this->isNeedToDoEventTypeBeanValid();
        $exp2 = $this->eventTypeContainsBeforehandNotifications();
        $exp3 = $this->eventTypeContainsOverdueNotifications();
        $exp4 = $this->isAssignedUserMustBeNotified();
        $exp5 = $this->isUsersDefinedForNotification();
//        $GLOBALS['log']->fatal('$exp1: ' . print_r($exp1, true));
//        $GLOBALS['log']->fatal('$exp2: ' . print_r($exp2, true));
//        $GLOBALS['log']->fatal('$exp3: ' . print_r($exp3, true));
//        $GLOBALS['log']->fatal('$exp4: ' . print_r($exp4, true));
//        $GLOBALS['log']->fatal('$exp5: ' . print_r($exp5, true));

        return $exp1 && ($exp2 || $exp3) && ($exp4 || $exp5);
    }

    protected function isNeedToDoEventTypeBeanValid(){
        return !empty($this->needToDoEventType);
    }

    protected function eventTypeContainsBeforehandNotifications(){
        return !empty($this->needToDoEventType->notify_beforehand) && !empty($this->needToDoEventType->notify_beforehand_units) && !empty($this->needToDoEventType->notify_beforehand_duration);
    }

    protected function eventTypeContainsOverdueNotifications(){
        return !empty($this->needToDoEventType->notify_beforehand);
    }

    protected function isAssignedUserMustBeNotified(){
        if(empty($this->needToDoEventType->notify_assigned_user))
            return false;

        return !empty($this->needToDoEventType->assigned_user_id);
    }

    protected function isUsersDefinedForNotification(){
        return !empty($this->needToDoEventType->notify_users);
    }

    protected function createUserNotificationList(){
        $userIds = array();
        if($this->isUsersDefinedForNotification()){
            $notifyUsers = $this->needToDoEventType->decodeNotifyUsers();
            foreach($notifyUsers as $notifyUser){
                $userIds[$notifyUser['id']] = $notifyUser['id'];
            }
        }

        if($this->isAssignedUserMustBeNotified()){
//            $GLOBALS['log']->fatal('$userIds: ' . print_r($userIds, true));
            $userIds[$this->needToDoEvent->assigned_user_id] = $this->needToDoEvent->assigned_user_id;
        }

//        $GLOBALS['log']->fatal('$userIds: ' . print_r($userIds, true));
        $this->creteRecordsForNotification($userIds);
    }

    protected function creteRecordsForNotification(array $userIds){
        global $timedate, $current_user;
        $db = \DBManagerFactory::getInstance();
        $currentDatetime = $timedate->nowDb();
        $currentUserId = $current_user->id;
        $eventId = $this->needToDoEvent->id;
        $notifyUsersData = $this->getNTDVNotifyUsersRecord($eventId);

        $this->cleanUpOldNTDVNotifyUsersData($eventId);
        foreach($userIds as $userId) {
            $notifiedFlag = $this->getNotifyBeforehandDateFlag($notifyUsersData, $userId);
            $overdueNotifiedFlag = $this->getNotifyOverdueDateFlag($notifyUsersData, $userId);

            $newId = \Sugarcrm\Sugarcrm\Util\Uuid::uuid1();
            $sql = "
                INSERT INTO px_ntdv_notify_users 
                      (id, date_entered, created_by, description, event_id, user_id, notified, overdue_notified)
                    VALUES 
                      (
                        '" . $newId . "',
                        '" . $currentDatetime . "',
                        '" . $currentUserId . "',
                        '',
                        '" . $eventId . "',
                        '" . $userId . "',
                        " . $notifiedFlag . ",
                        " . $overdueNotifiedFlag . "
                      )
            ";
            $db->query($sql);
        }
    }

    protected function getNTDVNotifyUsersRecord($eventId){
        $db = \DBManagerFactory::getInstance();
        $sql = "
            SELECT id, event_id, user_id, notified, overdue_notified FROM px_ntdv_notify_users
            WHERE 
              event_id='" . $eventId . "'
        ";
        $rs = $db->query($sql);
        $ids = array();
        while($row = $db->fetchByAssoc($rs)){
            $ids[$row['user_id']] = $row;
        }
        return $ids;
    }

    protected function cleanUpOldNTDVNotifyUsersData($eventId){
        $db = \DBManagerFactory::getInstance();
        $sql = "
            DELETE FROM px_ntdv_notify_users
            WHERE event_id='" . $eventId . "'
        ";
        $db->query($sql);
    }

    protected function getNotifyBeforehandDateFlag($notifyUsersData, $userId){
        //if event doesn't contains beforehand notifications
        if(!$this->eventTypeContainsBeforehandNotifications()){
            return self::NOTIFICATION_OFF_STATUS;
        }
//        elseif(empty($notifyUsersData[$userId])){
//            return self::NOTIFICATION_ON_STATUS;
//        }
        //if notification wasn't set, so we keep it anf send anyway
        elseif(isset($notifyUsersData[$userId]['notified']) && $notifyUsersData[$userId]['notified'] == self::NOTIFICATION_ON_STATUS){
            return self::NOTIFICATION_ON_STATUS;
        }
        //if notification wasn't set, there is no record at all in table px_ntdv_notify_users
        //OR if date was changed
        elseif(empty($notifyUsersData[$userId]) || $this->needToDoEvent->notify_date != $this->needToDoEvent->fetched_row['notify_date']) {
            //if notification date was changed
            global $timedate;
            $eventNotifyDate = $this->needToDoEvent->notify_date;
            $eventNotifyDatetime = $timedate->fromDb($eventNotifyDate);
            $currentDatetime = $timedate->getNow();

            if (empty($eventNotifyDatetime) || empty($currentDatetime)) {
                //if notification date are undefined - we consider notification sent
                return self::NOTIFICATION_COMPLETED_STATUS;
            }
//            $GLOBALS['log']->fatal('1EventsDate: ' . print_r($eventNotifyDatetime->getTimestamp(), true));
//            $GLOBALS['log']->fatal('1$currentDate: ' . print_r($currentDatetime->getTimestamp(), true));

            $eventNotifyTimestamp = $eventNotifyDatetime->getTimestamp();
            $currentTimestamp = $currentDatetime->getTimestamp();
            if ($eventNotifyTimestamp >= $currentTimestamp) {
                //if notification was sent, but date was changed. New date is greater then current date.
                return self::NOTIFICATION_ON_STATUS;
            }
            return self::NOTIFICATION_COMPLETED_STATUS;
        }
        //if notification was sent and date didn't change
        return self::NOTIFICATION_COMPLETED_STATUS;
    }

    protected function getNotifyOverdueDateFlag($notifyUsersData, $userId){
        //if event doesn't contains overdue notifications
        if(!$this->eventTypeContainsOverdueNotifications()){
            return self::NOTIFICATION_OFF_STATUS;
        }
//        //if notification wasn't set, there is no record at all in table px_ntdv_notify_users
//        elseif(empty($notifyUsersData[$userId])){
//            return self::NOTIFICATION_ON_STATUS;
//        }
        //if overdue notification wasn't set, so we keep it anf send anyway
        elseif(isset($notifyUsersData[$userId]['overdue_notified']) && $notifyUsersData[$userId]['overdue_notified'] == self::NOTIFICATION_ON_STATUS){
            return self::NOTIFICATION_ON_STATUS;
        }
        //if notification wasn't set, there is no record at all in table px_ntdv_notify_users
        //OR if date was changed
        elseif(empty($notifyUsersData[$userId]) || $this->needToDoEvent->control_date != $this->needToDoEvent->fetched_row['control_date']) {
            //if overdue notification date was changed
            global $timedate;
            $eventControlDate = $this->needToDoEvent->control_date;
            $eventControlDatetime = $timedate->fromDb($eventControlDate);
            $currentDatetime = $timedate->getNow();

            if (empty($eventControlDatetime) || empty($currentDatetime)) {
                //if notification date are undefined - we consider notification sent
                return self::NOTIFICATION_COMPLETED_STATUS;
            }
//            $GLOBALS['log']->fatal('2EventsDate: ' . print_r($eventControlDatetime->getTimestamp(), true));
//            $GLOBALS['log']->fatal('2$currentDate: ' . print_r($currentDatetime->getTimestamp(), true));

            $eventControlTimestamp = $eventControlDatetime->getTimestamp();
            $currentTimestamp = $currentDatetime->getTimestamp();
            if ($eventControlTimestamp >= $currentTimestamp) {
                //if notification was sent, but date was changed. New date is greater then current date.
                return self::NOTIFICATION_ON_STATUS;
            }
            return self::NOTIFICATION_COMPLETED_STATUS;
        }
        //if notification was sent and date didn't change
        return self::NOTIFICATION_COMPLETED_STATUS;
    }

}