<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers\Notification\Entities;

use \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers\SugarUrlGenerator;
use \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers\ConversationParticipants;

class Comment{

    protected $commentBean = null;
    protected $needToDoEventsBean;
    protected $urlGenerator = null;

    public function __construct(\PX_NeedToDoViewerComments $comment, SugarUrlGenerator $urlGenerator){
        $this->commentBean = $comment;
        $this->urlGenerator = $urlGenerator;
    }

    /**
     * @return ConversationParticipants
     */
    public function getConversationParticipants(){
        return $this->commentBean->getConversationParticipants();
    }

    public function setUrlGenerator(SugarUrlGenerator $urlGenerator){
        $this->urlGenerator = $urlGenerator;
    }

    public function setCreatedUser($createdUser = null){
        if(!empty($createdUser)){
            $this->commentBean->created_by_bean = $createdUser;
        }else{
            global $current_user;
            if($this->commentBean->created_by == $current_user->id){
                $this->commentBean->created_by_bean = $current_user;
            }else{
                $this->commentBean->created_by_bean = \BeanFactory::retrieveBean('Users', $this->commentBean->created_by);
            }
        }
    }

    public function getCreatedUser(){
        if(!isset($this->commentBean->created_by_bean)){
            $this->setCreatedUser();
        }
        return $this->commentBean->created_by_bean;
    }

    public function getParentBean(){
        $parentBean = null;
        if($this->isReply()){
            $parentBean = $this->commentBean->getParentBean();
        }
        return $parentBean;
    }

    public function getParentBeans(){
        if($this->isReply()){
            return $this->_getParentCommentBeans($this->commentBean);
        }
        return [];
    }

    protected function _getParentCommentBeans($comment, $beanList = []){
        if($comment->parent_type == 'PX_NeedToDoEvents' || empty($comment->parent_id))
            return $beanList;

        $beanList[] = $parent = $comment->getParentBean();
        return $this->_getParentCommentBeans($parent, $beanList);
    }

    public function getNeedToDoEventBean(){
        if(!isset($this->needToDoEventsBean))
            $this->needToDoEventsBean = $this->_getNeedToDoEventBean($this->commentBean);
        return $this->needToDoEventsBean;
    }

    private function _getNeedToDoEventBean($comment, $iteration = 0){
        $parent = $comment->getParentBean();
        if(empty($parent))
            return null;
        else if($parent->module_name == 'PX_NeedToDoEvents'){
            return $parent;
        }
        else if($iteration >= 15)
            return null;

        return $this->_getNeedToDoEventBean($parent, $iteration + 1);
    }

    public function getContextBean(){
        if(!empty($this->needToDoEventsBean)){
            $this->needToDoEventsBean = \BeanFactory::getBean($this->needToDoEventsBean->parent_type, $this->needToDoEventsBean->parent_id);
        }
    }

    public function getMessageTags(){
        $commentData = $this->commentBean->getCommentDataAsArray();

        if(!empty($commentData['tags']))
            return $commentData['tags'];

        return array();
    }

    public function isReply(){
        return $this->commentBean->parent_type == 'PX_NeedToDoViewerComments' && !empty($this->commentBean->parent_id);
    }

    public function getParentAuthorId(){
//        if(!$this->isReply())
//           return null;
        $parentBean = $this->getParentBean(); //commentBean->getParentBean();
        return !empty($parentBean) ? $parentBean->created_by : null;
    }

    public function getAuthorId(){
        return $this->commentBean->created_by;
    }

    public function getMessageText(){
        $commentData = $this->commentBean->getCommentDataAsArray();
        $commentText = $commentData['value'];
        $links = array();
        $tagRegExp = '/@\[([\w]+):([\d\w\-]+):(.+?)\]/';

        if (!preg_match($tagRegExp, $commentText))
            return $commentText;

        preg_match_all($tagRegExp, $commentText, $links);

        foreach($links[0] as $index => $link_string){
            $commentText = str_replace($link_string, $this->getLinkTagsForText($links, $index), $commentText);
        }

        return $commentText;
    }

    protected function getLinkTagsForText($links, $index){
        $module = $links[1][$index];
        $id = $links[2][$index];
        $name = $links[3][$index];
        $url = $this->urlGenerator->getSugarRecordUrl($module, $id);

        return "<a href='" . $url ."' target='_blank'>$name</a>";
    }

    public function getCreatedDate(){
        global $timedate;
        $date = $this->commentBean->date_entered;
        $user = $this->getCreatedUser();

        if($timedate->fromUserType($date, $user)){
            return $timedate->to_db($date);
        }
        return $date;
    }

}