<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers\Notification\MessageComposer;

use \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers\SugarUrlGenerator;

class MessageComposer
{
    protected $messageTemplate;
    protected $event;
    protected $urlGenerator = null;

    public function __construct(SugarUrlGenerator $urlGenerator)
    {
        $this->messageTemplate = new MessageTemplate();
        $this->urlGenerator = $urlGenerator;
    }

    public function getTitle($user, $event)
    {
        $template = $this->messageTemplate->getTitleTemplate($user, $event->getLangKey());
        $data = $event->getData();
        $createdBy = $this->getEntityName($data['CREATED_BY']);
        return str_replace(['{{CREATED_BY}}'], [$createdBy], $template);
    }

    public function getText($user, $event)
    {
        $template = $this->messageTemplate->getBodyTemplate($user, $event->getLangKey());
        $data = $event->getData();
        $createdBy = $this->getEntityName($data['CREATED_BY']);
        $commentText = $data['COMMENT_TEXT'];
        $dateEntered = $this->getDateFormatted($data['DATE_ENTERED']);
        $parentLink = $this->getEntityName($data['NEED_TO_DO_EVENT_BEAN']);
        $text = str_replace(
            ['{{COMMENT_TEXT}}', '{{DATE_ENTERED}}', '{{CREATED_BY}}', '{{PARENT_LINK}}'],
            [$commentText, $dateEntered, $createdBy, $parentLink],
            $template
        );
        return $text;
    }

    protected function getEntityName($bean, $user = null)
    {
        if (empty($bean)) {
            return '';
        } else if ($bean instanceof \Person) {
            return $bean->full_name;
        } else {
            return $bean->name;
        }
    }

    protected function getDateFormatted($date, $user = null)
    {
        if (empty($date)) {
            return '';
        }
        global $timedate;
        return $timedate->to_display_date_time($date, true, true, $user);
    }

}