<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers\Notification;

use \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers\Notification\Entities\Comment;
use \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers\Notification\MessageComposer\MessageComposer;

class NeedToDoViewerCommentNotifier
{
    protected $urlGenerator = null;
    protected $commentEntity = null;
    protected $eventsList = null;
    protected $messageComposer = null;
    protected $recipientsIds = [];
    protected $recipients = [];

    protected $notifiedUserIds = [];

    public const NOTIFICATION_SEVERITY = 'information';

    public function __construct()
    {
        $this->urlGenerator = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers\SugarUrlGenerator();
    }

    public function init($comment)
    {
        $this->initCommentEntity($comment);
        $this->initMessageComposer();
        $this->initEventsList();
        $this->flashNotifiedUserList();
    }

    protected function initCommentEntity($entity)
    {
        $this->commentEntity = new Comment($entity, $this->urlGenerator);
    }

    protected function initMessageComposer()
    {
        $this->messageComposer = new MessageComposer($this->urlGenerator);
    }

    protected function initEventsList()
    {
        $eventSequence = new EventsList();
        $this->eventsList = $eventSequence->initList($this->commentEntity)->getList();
    }

    public function notify()
    {
        foreach ($this->eventsList as $event) {
            $this->notifyAboutEvent($event);
        }

    }

    protected function notifyAboutEvent($event)
    {
        global $current_user;
        $recipientIds = $event->getRecipientIds();
        foreach ($recipientIds as $userId) {
            if ($current_user->id == $userId) {
                continue;
            }
            if (!$this->isUserNotified($userId)) {
                $this->createNotification($userId, $event);
                $this->markUserAsNotified($userId);
            }
        }
    }

    protected function isUserNotified($userId)
    {
        return isset($this->notifiedUserIds[$userId]);
    }

    protected function markUserAsNotified($userId)
    {
        $this->notifiedUserIds[$userId] = $userId;
    }

    protected function flashNotifiedUserList()
    {
        $this->notifiedUserIds = [];
    }

    protected function createNotification($userId, $event)
    {
        if (!isset($this->recipients[$userId])) {
            $this->recipients[$userId] = \BeanFactory::getBean('Users', $userId, array('strict_retrieve' => true));
        }
        if (empty($this->recipients[$userId])) {
            return;
        }
        $title = $this->messageComposer->getTitle($this->recipients[$userId], $event);
        $text = $this->messageComposer->getText($this->recipients[$userId], $event);
        $needToDoEventBean = $this->commentEntity->getNeedToDoEventBean();
        if (empty($needToDoEventBean) || empty($needToDoEventBean->id)) {
            return;
        }
        $notification = \BeanFactory::getBean('Notifications');
        $notification->name = $title;
        $notification->description = $text;
        $notification->parent_type = 'PX_NeedToDoEvents';
        $notification->parent_id = $needToDoEventBean->id;
        $notification->assigned_user_id = $userId;
        $notification->is_read = 0;
        $notification->severity = self::NOTIFICATION_SEVERITY;
        $notification->save();
    }
}