<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License;

use Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\JWT\Namshi\JOSE\SimpleJWS;
use Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Exceptions\InvalidLicenseException;

class Decryptor{

    /**
     * @var string
     * Path to the file that contains Public Key value
     */
    protected $publicKeyFilePath;

    /**
     * @var string
     * Public Key value
     */
    protected $publicKey;

    /**
     * @var string
     * The algorithm that was used for signing JWT
     */
    protected $jwtAlgorithm = 'RS256';

    /**
     * @param $publicKeyFilePath
     */
    public function __construct($publicKeyFilePath){
        $this->init($publicKeyFilePath);
    }

    /**
     * Init's public key
     *
     * @param $publicKeyFilePath
     */
    protected function init($publicKeyFilePath){
        $this->publicKeyFilePath = $publicKeyFilePath;
        $this->loadPublicKey();
    }

    /**
     * Reads public key from file.
     */
    protected function loadPublicKey(){
        if(!\SugarAutoLoader::fileExists($this->publicKeyFilePath)){
            $this->publicKey = '';
        }

        require $this->publicKeyFilePath;
        $this->publicKey = empty($publicKey) ? '' : $publicKey;
    }

    /**
     * Decrypts token
     *
     * @param string $token - Encrypted license token (String representing the JWT)
     * @return array - Decrypted license token payload (Payload of the JWT)
     * @throws InvalidLicenseException
     */
    public function decrypt($token){
        $tokenPayload = array();
        $this->checkKey();
        $this->checkToken($token);

        try {
            $jws = SimpleJWS::load($token);
            if ($jws->isValid($this->publicKey, $this->jwtAlgorithm)) {
                $tokenPayload = $jws->getPayload();
            }
        }catch(\Exception $e){
            throw new InvalidLicenseException($e->getMessage());
        }

        $this->checkTokenPayload($tokenPayload);
        return $tokenPayload;
    }

    protected function checkKey(){
        if(empty($this->publicKey)){
            $this->throwError('ERROR_LICENSE_INVALID_SSH_PUBLIC_KEY');
        }
    }

    protected function checkToken($token){
        if(empty($token)){
            $this->throwError('ERROR_LICENSE_EMPTY_TOKEN');
        }
    }

    protected function checkTokenPayload($tokenPayload){
        if(empty($tokenPayload)){
            $this->throwError('ERROR_LICENSE_TOKEN_PARSE_FAILURE');
        }
    }

    protected function throwError($labelKey){
        $label = \translate($labelKey, 'PX_NeedToDoEvents');
        throw new InvalidLicenseException($label);
    }
}