<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Validations\Decorators;

use Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Validations\Validation;
use Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Exceptions\InvalidLicenseException;

class ProductValidationDecorator implements Validation
{

    /**
     * @var Validation
     */
    protected $validation;

    /**
     * @var array - Decoded License Token
     */
    protected $tokenPayload;

    public const CURRENT_PRODUCT_NAME = 'NeedToDo';

    /**
     * Product Name Token payload key
     */
    public const PK_PRODUCT_NAME = 'product_name';

    public function __construct(Validation $validation)
    {
        $this->validation = $validation;
        $this->tokenPayload = $this->validation->getTokenPayload();
    }

    public function getTokenPayload()
    {
        return $this->validation->getTokenPayload();
    }

    public function validateLicense()
    {
        $this->validation->validateLicense();
        $this->checkProductName();
    }

    public function validateUser($user)
    {
        $this->validation->validateUser($user);
    }

    public function getLicenseType()
    {
        return $this->validation->getLicenseType();
    }

    public function getExpireDate()
    {
        return $this->validation->getExpireDate();
    }

    public function getCurrentUsers()
    {
        return $this->validation->getCurrentUsers();
    }

    public function getUsersLimit()
    {
        return $this->validation->getUsersLimit();
    }

    public function clearCachedData()
    {
        $this->validation->clearCachedData();
    }

    /**
     * Check whether Product Name is equal to NeedToDo
     * @throws InvalidLicenseException
     */
    private function checkProductName()
    {
        $productNamePayloadValue = $this->tokenPayload[self::PK_PRODUCT_NAME];

        if ($productNamePayloadValue != self::CURRENT_PRODUCT_NAME) {
            $this->throwException('ERROR_INVALID_LICENSE_TOKEN');
        }
    }

    /**
     * @param $label - Sugar error label key
     * @throws InvalidLicenseException
     */
    private function throwException($label)
    {
        $message = translate($label, 'PX_NeedToDoEvents');
        throw new InvalidLicenseException($message);
    }

}