({
    extendsFrom: "BaseField",
    rowId: 0,

    formRowTemplate: null,

    hasSystemName : false,

    notifiedOfParseError: false,

    fields: [],

    initialize: function(arguments){
        this._super("initialize", [arguments]);

        this.events = _.extend(this.events, {
            'click [ntdv-eventtype-relate-action=add]': 'addRow',
            'click [ntdv-eventtype-relate-action=remove]': 'removeRow',
            'click .btn.inactive[ntdv-eventtype-relate-action=add-link]': 'addRelateToTargetChainByButton',
            'click .btn[ntdv-eventtype-relate-action=add-link]:not(.inactive)': 'removeRelateToTargetChainByButton',
        });

        this.formRowTemplate = app.template.getField('parent-relate-list', 'parent-relate-item', this.module);

        this.fields = [];
        this.notifiedOfParseError = false;
    },

    bindDataChange: function() {
        if (this.model) {
            this.model.on(/*"change:lb_template_name " + */"change:" + this.name, function(model, value) {
                this.render();
            }, this);
        }
    },

    _render: function(values) {
        this.rowId = 0;
        this._loadTemplate();
        this.$el.html(this.template(this) || '');

        this.renderFields(values);
        // this.displayInsertButton();

        this.changeStylesForView();
    },

    changeStylesForView: function(){
        //this.action
        if(this.tplName == 'detail'){
            var elements = this.$el.find('.ntdv-relate-element-part');
            _.each(elements, function(element){
                var jelement = $(element);
                jelement.attr(
                    'class',
                    jelement.attr('class').replace(/\bspan\d+\b/g, '')
                );

                var linkInfo = jelement.data('container');
                var ntdvElement = jelement.closest('.ntdv-eventtype-relate-element');
                var linkSeparator = ntdvElement.find('.detail-separator[data-next-link="' + linkInfo + '"]');
                if(linkSeparator.length){
                    if(jelement.hasClass('invisible')) {
                        linkSeparator.hide();
                    }else{
                        linkSeparator.show();

                    }
                }

            }, this);
        }
    },

    displayInsertButton: function(){
        if(this.disposed || _.isEmpty(this.$el) || !this.$el.find('.ntdv-eventtype-relate-actions').length)
            return;

        if(this.action == 'detail')
            this.$el.find('.ntdv-eventtype-relate-actions').hide();
        else
            this.$el.find('.ntdv-eventtype-relate-actions').show();
    },

    renderFields: function(values){
        var defs = values || this.model.get(this.name);

        if(!_.isEmpty(defs) && _.isString(defs)){
            try{
                defs = JSON.parse(defs);
            }catch(err){
                defs = null;
                var errorMsg = app.lang.get('LBL_CONTEXT_PARSE_ERROR', 'PX_FlowchartTemplates');
                app.logger.warn(errorMsg);
                if(!this.notifiedOfParseError) {
                    this.notifiedOfParseError = true;
                    app.alert.show('context_parse_error', {
                        level: 'warning',
                        messages: errorMsg
                    });
                }
            }
        }

        if(_.isEmpty(defs) || (_.isArray(defs) && defs.length <= 0) || (_.isString(defs) && defs == '[]')){
            this.renderRow(null, {});
            return;
        }

        var self = this;
        _.each(defs, function(data, key){
            self.renderRow(null, data);
        });
    },

    getRowSequenceNumber: function(data){
        if(_.isUndefined(data['key'])){
            return ++this.rowId;
        }

        if(data['key'] > this.rowId)
            this.rowId = data['key'];

        return data['key'];
    },

    addRelateToTargetChainByButton: function(e){
        var currentButton = $(e.currentTarget);
        var levelId = currentButton.data('level-id');

        var $row = this.$(e.currentTarget).closest('[data-container=parent-relate-item]');
        this.addRelateToTargetChain($row, {}, levelId);
    },

    removeRelateToTargetChainByButton: function(e){
        var currentButton = $(e.currentTarget);
        var levelId = currentButton.data('level-id');

        var $row = this.$(e.currentTarget).closest('[data-container=parent-relate-item]');
        var relateElementList = $row.find('.ntdv-eventtype-relate-element');
        var newLevelId = parseInt(levelId) + 1;
        var children = relateElementList.children('.ntdv-relate-element-part');

        for(var newLevelId = parseInt(levelId) + 1; newLevelId <= 3; newLevelId++) {
            var targetRelateElement = relateElementList.find('.ntdv-relate-element-part[data-container="link-level-' + newLevelId + '"]');
            if (targetRelateElement.length) {
                targetRelateElement.addClass('invisible');
                targetRelateElement.find('.btn.link-level').addClass('inactive');
            }
        }

        currentButton.addClass('inactive');
        this.changeElemItemsWidth(children, (levelId - 1));
    },



    addRelateToTargetChainByLevelId: function($row, data, levelId){
        this.addRelateToTargetChain($row, data, levelId);
    },

    addRelateToTargetChain: function($row, data, levelId){
        var addLinkButton = $row.find('button[data-level-id="' + levelId + '"]');
        var relateElementList = $row.find('.ntdv-eventtype-relate-element');
        var newLevelId = parseInt(levelId) + 1;

        var children = relateElementList.children('.ntdv-relate-element-part');

        this.addNextLevelLink($row, relateElementList, newLevelId, data);
        this.changeElemItemsWidth(children, levelId);
        if(!_.isEmpty(addLinkButton)) {
            addLinkButton.removeClass('inactive');
        }
    },

    addNextLevelLink: function($row, relateElementList, newLevelId, data){
        data = data || {};
        var rowId = $row.data('key');
        var targetRelateElement = relateElementList.find('.ntdv-relate-element-part[data-container="link-level-' + newLevelId + '"]');
        if(targetRelateElement.length){
            targetRelateElement.removeClass('invisible');
        }

        if(!targetRelateElement.find('span.edit').length){
            this.renderLinkField($row, data, rowId, newLevelId);
        }
    },

    changeElemItemsWidth(elements, levelId){
        var spanClassName = 'span6';
        if(levelId == 1){
            spanClassName = 'span4'
        }else if(levelId == 2){
            spanClassName = 'span3'
        }

        _.each(elements, function(element){
            $(element).attr(
                'class',
                $(element).attr('class').replace(/\bspan\d+\b/g, '')
            );
            // $(element).removeClass('span6 span4 span3');
            $(element).addClass(spanClassName);
        }, this);
    },

    addRow: function(e){
        var $row;
        if (e) {
            $row = this.$(e.currentTarget).closest('[data-container=parent-relate-item]');
            $row.after(this.formRowTemplate({hasSystemName: this.hasSystemName}));
            $row = $row.next();
        }

        this.renderRow($row, {});

        // var rowInput = $row.find('input[type="text"]');
        // if(rowInput.length)
        //     rowInput.focus();
    },

    removeRow: function(e){
        var $row = this.$(e.currentTarget).closest('[data-container=parent-relate-item]');

        this.removeModelValues($row);

        $row.remove();
        if (this.$('[data-container=parent-relate-item]').length === 0) {
            this.addRow();
        }
    },

    removeModelValues: function($row){
        var data = $row.data(),
            linkLevelOneName = data['linkLevel1'];
            // ,keyName = data['value'];

        this.model.set(linkLevelOneName, '');
        // this.model.set(keyName, '');
    },

    renderRow: function($row, data){
        $row = $row || $(this.formRowTemplate()).appendTo(this.$el);
        data = data || {};

        var rowId = this.getRowSequenceNumber(data);
        $row.data('key', rowId);

        this.renderLinkLabel($row, data, rowId);
        this.renderLinkField($row, data, rowId, 1);
        if(!_.isEmpty(data['linkLevel2'])) {
            this.addRelateToTargetChain($row, data, 1);

            // this.renderLinkField($row, data, rowId, 2);
            if(!_.isEmpty(data['linkLevel3'])) {
                this.addRelateToTargetChain($row, data, 2);
                // this.renderLinkField($row, data, rowId, 3);
            }
        }
        // this.renderValueAttribute($row, data, rowId);
    },

    renderLinkLabel: function($row, data, rowId){
        var field, fieldContainer, fieldEl,
            fieldName = this.getLinkLabelFieldName(rowId),
            fieldValue = data.linkLabel || '',
            fieldPlaceholderLabelKey = 'LBL_PARENT_LINK_LABEL',
            selectPlaceholderLabel = "Set %FIELD_PLACEHOLDER%...".replace('%FIELD_PLACEHOLDER%', app.lang.get(fieldPlaceholderLabelKey, this.module));

        this.model.set(fieldName, fieldValue);
        var fieldDefs = _.extend({
            name: fieldName,
            type: 'varchar',
            // type: 'template-attributes-enum',
            parentFieldName: this.name,
            placeholder: selectPlaceholderLabel,
            // argumentPart: 'template-attribute',
            disabled: true,
            layout: this
        }, this.def.options.linkLabel || {});
        // return;
        field = this.createField(this.model, fieldDefs);
        fieldContainer = $row.find('[data-container=link-label]');
        fieldEl = $(field.getPlaceholder().string);
        fieldEl.appendTo(fieldContainer);
        this.renderField(field, fieldEl);
        this.fields.push(field);
    },

    renderLinkField: function($row, data, rowId, linkLevel){
        rowId = rowId || $row.data('key');
        var field, fieldContainer, fieldEl,
            fieldName = this.getLinkFieldName(rowId, linkLevel),
            fieldValue = data['linkLevel'+linkLevel] || '',
            fieldLabel = 'LBL_PARENT_LINK_NAME';

        this.model.set(fieldName, fieldValue);
        var fieldDefs = _.extend({
            name: fieldName,
            type: 'enum',
            // type: 'template-attributes-enum',
            parentFieldName: this.name,
            selectPlaceholder: fieldLabel,
            dependOn: this.getPreviousRelateChainNames(rowId, linkLevel),
            // argumentPart: 'template-attribute',
            disabled: true,
            layout: this
        }, this.def.options['linkLevel'+linkLevel] || {});
        // return;
        field = this.createField(this.model, fieldDefs);
        fieldContainer = $row.find('[data-container=link-level-' + linkLevel + ']');
        if(fieldContainer.hasClass('invisible')){
            fieldContainer.removeClass('invisible');
        }
        fieldEl = $(field.getPlaceholder().string);
        fieldEl.prependTo(fieldContainer);
        this.renderField(field, fieldEl);
        this.fields.push(field);
    },

    getPreviousRelateChainNames: function(rowId, linkLevel){
        var relateChainNames = [];
        for(var i=0; i < linkLevel; i++){
            if(!i){
                relateChainNames.push('target_module');
            }else {
                relateChainNames.push(this.getLinkFieldName(rowId, i));
            }
        }

        return relateChainNames;
    },

    renderValueAttribute: function($row, data, rowId){
        var fieldDefs, field,
            fieldContainer, fieldEl,
            fieldKeyName = this.getAttributeSystemName(rowId),
            fieldKeyValue = data.name || '';

        $row.data('value', fieldKeyValue);
        this.model.set(fieldKeyName, fieldKeyValue);

        fieldDefs = _.extend({
            name: fieldKeyName,
            type: 'enum',
            parentFieldName: this.name,
            argumentPart: 'target-field',
            disabled: true,
            layout: this
        }, this.def.options.name || {});

        field = this.createField(this.model, fieldDefs);
        fieldContainer = $row.find('[data-container=ntdv-eventtype-relate-key]');
        fieldEl = $(field.getPlaceholder().string);
        fieldEl.appendTo(fieldContainer);

        this.renderField(field, fieldEl);
        this.fields.push(field);
    },

    getAttributeSystemName: function(key){
        return 'attr_sys_name' + key
    },

    getLinkLabelFieldName: function(key){
        return 'link_label_' + key
    },

    getLinkFieldName: function(key, linkLevel){
        return 'link_level' + linkLevel + '_' + key
    },

    createField: function(model, def) {
        var obj = {
            meta: {
                view: 'edit'
            },
            def: def,
            model: model,
            context: app.controller.context,
            viewName: this.action,
            view: this.view
        };

        model.fields[def.name] = def;

        var field = app.view.createField(obj);
        field.action = this.action;
        return field;
    },

    renderField: function(field, $fieldEl) {
        field.setElement($fieldEl);
        try {
            field.render();
        } catch (e) {
            app.logger.error("Failed to render " + field + " on " + this.view + "\n" + e);
            app.error.handleRenderError(this.view, '_renderField', field);
        }
    },

    toggle: function(evt) {
        this.$('[data-container=parent-relate-item]').html('');
        this.renderFields();
    },

    clearErrorDecoration: function () {
        _.each(this.fields, function(field, key) {
            if(_.isEmpty(field) || _.isEmpty(field.$el)){
                // delete this.fields[key];
            }else {
                field.clearErrorDecoration();
            }
        }, this);

        this._super('clearErrorDecoration');
    },

    beforeHandleSave: function(){
        this.model.set('target_entity_parent_list', this.getFormattedFieldValue());
    },

    getFieldData: function(){
        var $rows = this.$('[data-container=parent-relate-item]');
        var contextData = [];

        _.each($rows, function(row) {
            var argumentItem = this.getArgument($(row));
            if(/*!_.isEmpty(argumentItem.name) &&*/ !_.isEmpty(argumentItem.linkLevel1))
                contextData.push(argumentItem);
        }, this);

        return contextData;
    },

    getFormattedFieldValue: function(){
        var contextData = this.getFieldData();
        return JSON.stringify(contextData, null);
    },

    getArgument: function($row) {
        var argument = {},
            data = $row.data(),
            key = data['key'],
            // fieldKeyName = this.getAttributeSystemName(key);
            linkLabelName = this.getLinkLabelFieldName(key),
            linkLevel1Name = this.getLinkFieldName(key, 1),
            linkLevel2Name = this.getLinkFieldName(key, 2),
            linkLevel3Name = this.getLinkFieldName(key, 3);

        argument = {
            key: key,
            linkLabel: this.model.get(linkLabelName),
            linkLevel1: this.model.get(linkLevel1Name)
        };

        if(!_.isEmpty(linkLevel2Name) && !_.isEmpty(this.model.get(linkLevel2Name)) && (this.tplName == 'detail' || this.isLinkLevelSelectVisible($row, linkLevel2Name)) ){
            argument['linkLevel2'] = this.model.get(linkLevel2Name);
            if(!_.isEmpty(linkLevel3Name) && !_.isEmpty(this.model.get(linkLevel3Name)) && (this.tplName == 'detail' || this.isLinkLevelSelectVisible($row, linkLevel3Name)) ){
                argument['linkLevel3'] = this.model.get(linkLevel3Name);
            }
        }

        return argument;
    },

    isLinkLevelSelectVisible: function($row, linkFieldName){
        var fieldElement = $row.find('input[name="' + linkFieldName + '"]');
        if(!fieldElement.length){
            return false;
        }

        var relateElementPart = fieldElement.closest('.ntdv-relate-element-part');
        return !(relateElementPart.hasClass('invisible'));
    },

    _dispose: function() {
        _.each(this.fields, function(field){
            field.dispose();
        });
        this.fields = [];
        this._super('_dispose');
    },

    isFieldLengthValid: function(){
        if(_.isEmpty(this.model.get('target_entity_parent_list')))
            return true;

        var contextField = btoa(escape(this.model.get('target_entity_parent_list')));
        if(contextField.length >= 65535){
            return false;
        }
        return true;
    },

    getSymbolsAmountViolation: function(){
        var contextField = btoa(escape(this.model.get('target_entity_parent_list')));
        return contextField.length - 65535;
    },

    substringContextFieldLength: function(){
        var contextList = this.getFieldData(),
            trimmedContextList = [],
            sortedContextList = [],
            whiteListedKeyNames = [],
            newContextList = [];

        sortedContextList = _.sortBy(contextList, function(context){ return context.linkLevel1.length + context.linkLabel.length; });

        _.each(sortedContextList, function(context){
            var tempArgumentsJson = '';
            trimmedContextList.push(context);

            try{
                tempArgumentsJson = btoa(escape(JSON.stringify(trimmedContextList, null)));
                if(tempArgumentsJson.length <= 65535){
                    whiteListedKeyNames.push(context.name);
                }else{
                    return;
                }
            }catch(err){
                return;
            }
        }, this);


        if(!_.isEmpty(whiteListedKeyNames)){
            _.each(contextList, function(context, key){
                if(!_.isEmpty(context) && _.indexOf(whiteListedKeyNames, context.name) !== -1){
                    newContextList.push(context);
                }
            });
        }

        try{
            this.model.set('target_entity_parent_list', JSON.stringify(newContextList, null));
        }catch(err){
            this.model.set('target_entity_parent_list', '[]');
        }
    },

    getFieldsMeta: function(){
        var contextList = this.getFieldData(),
            fieldsMeta = [];

        _.each(contextList, function(contextItem){
            fieldsMeta.push({
                'name': 'parent_relate_' + contextItem.key
            });
        }, this);

        return fieldsMeta;
    },

})
