({
    extendsFrom: "RecordView",

    initialize: function(options) {
        this._super('initialize', [options]);

        this.checkCurrentUserLicenseAccess();
    },

    delegateButtonEvents: function() {
        this.context.on('button:index_button:click', this.indexClicked, this);
        this._super('delegateButtonEvents');
    },

    prepareCustomFields: function(){
        if(this.getField('target_entity_filter')) {
            this.getField('target_entity_filter').beforeHandleSave();
        }
        if(this.getField('lb_template_arguments')) {
            this.getField('lb_template_arguments').beforeHandleSave();
        }
        if(this.getField('target_entity_parent_list')) {
            this.getField('target_entity_parent_list').beforeHandleSave();
        }
        if(this.getField('notify_users')) {
            this.getField('notify_users').beforeHandleSave();
        }
    },

    handleSave: function() {

        this.prepareCustomFields();

        if(this.hasCoreEventTypeDataChanged()){
            app.alert.show('update_confirmation', {
                level: 'confirmation',
                messages: app.lang.get('LBL_UPDATE_CONFIRMATION', this.module),
                onConfirm: _.bind(function(){
                    this._super("handleSave");
                }, this),
                onCancel: _.bind(function() {

                }, this)
            });
        }else{
            this._super("handleSave");
        }
    },

    handleLockedFields: function() {
        var self = this;

        // Reset the locked field state
        this._setLockedFieldFlag(false);

        var lockedFields = this.model.get('locked_fields');

        // Loop and check locked field state of each field
        _.each(this.$('.record-lock-link-wrapper[data-name]'), function(el) {
            var $el = $(el);
            var fieldName = $el.data('name');

            // No field name, nothing to do
            if (fieldName == '') {
                return;
            }

            // Get the field object
            var field = this.getField(fieldName);

            // Is the current field locked?
            var isLocked = _.contains(lockedFields, fieldName);

            // Special handling for fieldsets
            if (field.fields && field.name != 'target_entity_filter' && field.name != 'target_entity_parent_list' && field.name != 'notify_users') {
                var hasLockedChildField = false;
                isLocked = true;
                _.each(field.fields, function(fieldSetField) {
                    // Some fieldsets have fields that are only for viewing, like the
                    // `copy` field on alternate addresses. Those should be filtered
                    // out of the fields list.
                    if (_.isUndefined(this.model.get(fieldSetField.name))) {
                        return;
                    }

                    var isChildLocked = _.contains(lockedFields, fieldSetField.name);
                    hasLockedChildField = hasLockedChildField || isChildLocked;

                    // A fieldset is locked when all of its actual fields are locked
                    if (!isChildLocked) {
                        isLocked = false;
                    }
                }, this);
            }

            // Set the flag that says if we have locked fields
            this._setLockedFieldFlag(this.hasLockedFields() || isLocked || hasLockedChildField);

            // Handle toggling the class
            $el.toggleClass('hide', !isLocked);
        }, this);

        // Show the locked field warning if there is one
        if (this.hasLockedFields()) {
            this.warnLockedFields();
        }
    },

    hasUnsavedChanges: function() {
        this.prepareCustomFields();
        return this._super('hasUnsavedChanges');
    },

    hasCoreEventTypeDataChanged: function(){
        var changedAttributes = this.model.changedAttributes(this.model.getSynced());

        if(_.find(changedAttributes, function(val, key){
            return key == 'target_module' || key == 'target_attribute_name' || key == 'target_entity_filter';
        })){
            return true;
        }

        return false;
    },

    indexClicked: function(){

        this.indexEventType();
        // app.alert.show('update_confirmation', {
        //     level: 'confirmation',
        //     messages: app.lang.get('LBL_INDEXATION_CONFIRMATION', this.module),
        //     onConfirm: _.bind(function(){
        //         this.indexEventType();
        //     }, this),
        //     onCancel: _.bind(function() {
        //
        //     }, this)
        // });
    },

    indexEventType: function(){
        var ids = [];
        ids.push(this.model.get('id'));

        var url = app.api.buildURL('PX_NeedToDoEventTypes/index');
        app.api.call('create', url, { eventTypeIds: ids }, {
            success: _.bind(function (response) {
                app.alert.show('ntdv_index_completed_successful', {
                    level: 'success',
                    messages: app.lang.get('LBL_INDEXATION_SUCCESSFUL', this.module),
                    autoClose: true,
                    autoCloseDelay: 5000,
                });
            }, this),
            error: _.bind(function (error) {
                app.alert.show('ntdv_index_completed_error', {
                    level: 'error',
                    messages: app.lang.get('LBL_INDEXATION_ERROR', this.module),
                });
            }, this),
            complete: _.bind(function(){
                app.alert.dismiss("ntdv_index_event_type");
            }, this)
        });

        app.alert.show('ntdv_index_event_type', {
            level: 'process',
            title: app.lang.get('LBL_LOADING'),
            autoClose: false
        });
    },

    checkCurrentUserLicenseAccess: function(callback){
        var url = app.api.buildURL('PX_NeedToDoEvents', 'license/validate');
        app.api.call('read', url, null, {
            error: _.bind(function (response) {
                app.alert.show("ntdv_license_validation_error", {
                    level: "warning",
                    messages: response.message
                });
            }, this)
        });
    }
})