<?php

require_once 'clients/base/api/FilterApi.php';

class PX_NeedToDoEventsFilterApi extends FilterApi
{

    public function registerApiRest()
    {
        return array(
            'filterModuleGet' => array(
                'reqType' => 'GET',
                'path' => array('PX_NeedToDoEvents', 'filter'),
                'pathVars' => array('module', ''),
                'method' => 'filterList',
                'jsonParams' => array('filter'),
            ),
            'filterModuleAll' => array(
                'reqType' => 'GET',
                'path' => array('PX_NeedToDoEvents'),
                'pathVars' => array('module'),
                'method' => 'filterList',
                'jsonParams' => array('filter'),
            ),
            'filterModuleAllCount' => array(
                'reqType' => 'GET',
                'path' => array('PX_NeedToDoEvents', 'count'),
                'pathVars' => array('module', ''),
                'jsonParams' => array('filter'),
                'method' => 'getFilterListCount',
            ),
            'filterModulePost' => array(
                'reqType' => 'POST',
                'path' => array('PX_NeedToDoEvents', 'filter'),
                'pathVars' => array('module', ''),
                'method' => 'filterList',
            ),
            'filterModulePostCount' => array(
                'reqType' => 'POST',
                'path' => array('PX_NeedToDoEvents', 'filter', 'count'),
                'pathVars' => array('module', '', ''),
                'method' => 'filterListCount',
            ),
            'filterModuleCount' => array(
                'reqType' => 'GET',
                'path' => array('PX_NeedToDoEvents', 'filter', 'count'),
                'pathVars' => array('module', '', ''),
                'method' => 'getFilterListCount',
            ),
        );
    }

    protected $filterFlexRelateMappings = array(
        'filter_object_type' => 'object_type',
    );

    public function filterListSetup(ServiceBase $api, array $args, $acl = 'list')
    {
        if (!empty($args['filter']) && is_array($args['filter'])) {
            $contextEntityFilter = $this->getContextRelateFilter($args['filter']);
            $args['filter'] = $this->replaceNonDBFlexRelateFieldsInFilter($args['filter']);
        }

        [$args, $q, $options, $seed] = parent::filterListSetup($api, $args, $acl);

        if (!empty($contextEntityFilter)) {
            $this->applyContextFilters($q, $contextEntityFilter);
        }

        return array($args, $q, $options, $seed);
    }

    protected function applyContextFilters(SugarQuery $q, array $contextEntityFilter)
    {
        if (!empty($contextEntityFilter)) {
            $this->applyContextFilter($q, $contextEntityFilter);
        }
    }

    protected function applyContextFilter(SugarQuery $q, array $filterList)
    {
        if (empty($filterList)) {
            return;
        }

        $orWhere = $q->where()->queryOr();
        foreach ($filterList as $key => $filter) {
            if (!empty($filter['parent_type']) && !empty($filter['parent_id'])) {
                $plJta = 'px_ntdv_parents_' . $key;
                $joinTableSQL = "(SELECT event_id, parent_type, parent_id FROM px_ntdv_parents WHERE relate_type='Target'  GROUP BY event_id, parent_type, parent_id)";
                $q->join[$plJta] = $q->joinTable($joinTableSQL, array('alias' => $plJta, 'joinType' => 'LEFT'));
                $q->join[$plJta]->on()
                    ->equalsField($plJta . '.event_id', 'px_needtodoevents.id')
                    ->equals($plJta . '.parent_type', $filter['parent_type'])
                    ->equals($plJta . '.parent_id', $filter['parent_id']);

                $andWhere = $orWhere->queryAnd();
                $andWhere->isNotEmpty($plJta . '.event_id');
            } else if (!empty($filter['filter_parent_type'])) {
                $plJta = 'px_ntdv_parents_' . $key;
                $joinTableSQL = "(SELECT event_id, parent_type FROM px_ntdv_parents WHERE relate_type='Target' GROUP BY event_id, parent_type)";
                $q->join[$plJta] = $q->joinTable($joinTableSQL, array('alias' => $plJta, 'joinType' => 'LEFT'));
                $q->join[$plJta]->on()
                    ->equalsField($plJta . '.event_id', 'px_needtodoevents.id')
                    ->equals($plJta . '.parent_type', $filter['filter_parent_type']);
                $q->where()->isNotEmpty($plJta . '.event_id');
            }
        }
    }

    protected function getContextRelateFilter(array $filterList)
    {
        $contextListFilters = array();

        foreach ($filterList as $key => $filter) {
            foreach ($filter as $fieldName => $filterDefinition) {
                if (!empty($fieldName) && ($fieldName == 'parent_type' || $fieldName == 'parent_id' || $fieldName == 'filter_parent_type')) {
                    $contextListFilters[$fieldName] = $filterDefinition;
                }
                if (!empty($fieldName) && $fieldName == '$and') {
                    $contextFilterItem = $this->getContextRelateFilter($filterDefinition);
                    if (!empty($contextFilterItem) && !empty($contextFilterItem['parent_type']) && !empty($contextFilterItem['parent_id'])) {
                        $contextListFilters[] = $contextFilterItem;
                    }
                }
            }
        }
        return $contextListFilters;
    }

    protected function replaceNonDBFlexRelateFieldsInFilter($filterList)
    {
        foreach ($filterList as $key => $filter) {
            foreach ($filter as $fieldName => $filterDefinition) {
                if (!empty($this->filterFlexRelateMappings[$fieldName])) {
                    $realName = $this->filterFlexRelateMappings[$fieldName];
                    $filterList[$key][$realName] = $filterDefinition;
                    unset($filterList[$key][$fieldName]);
                }
            }
        }

        return $filterList;
    }

    protected function formatBean(ServiceBase $api, array $args, SugarBean $bean, array $options = array())
    {
        $data = parent::formatBean($api, $args, $bean, $options);
        if (!empty($data['comment_count']) && $data['comment_count'] > 0) {
            $data['has_new_comments'] = $bean->hasNewComments($data['id']);
        }

        return $data;
    }

}