({
    extendsFrom: "FilterFilterDropdownView",

    /**
     * Get the list of filters to fill the dropdown
     * @return {Array}
     */
    getFilterList: function() {
        var filters = [];
        if (this.layout.canCreateFilter()) {
            filters.push({id: "create", text: app.lang.get(this.labelCreateNewFilter)});
        }
        if (this.layout.filters.collection.get('all_records') && this.labelAllRecordsFormatted) {
            this.layout.filters.collection.get('all_records').set('name',  this.labelAllRecordsFormatted);
            this.layout.filters.collection.sort();
        }
        // This flag is used to determine when we have to add the border top (to separate categories)
        var firstNonEditable = false;

        var contextDashletType = this.getContextDashletType();
        this.layout.filters.collection.each(function(model) {

            var filterTemplate = model.get('filter_template') || model.get('filter_definition');
            if(contextDashletType != 'need-to-do-events-dashlet' || this.validateFilterDefinition(filterTemplate)){
                var opts = {
                    id: model.id,
                    text: this.layout.filters.collection._getTranslatedFilterName(model)
                };
                if (model.get("editable") === false && !firstNonEditable) {
                    opts.firstNonUserFilter = true;
                    firstNonEditable = true;
                }
                filters.push(opts);
            }

        }, this);

        return filters;
    },

    getContextDashletType: function(){
        var contextDashletModel = this.context.get('model');
        if(!_.isEmpty(contextDashletModel)){
            return contextDashletModel.get('type')
        }

        return '';
    },

    validateFilterDefinition: function(filterDefinition){
        var contextModule = 'Home', contextLayout;

        if(!_.isEmpty(this.context['parent'])) {
            contextModule = this.context.parent.get('module');
            contextLayout = this.context.parent.get('layout');
        }

        if(this.hasFilterForField(filterDefinition, 'filter_target_list_id'))
            return false;
        else if(contextModule !== 'Home'){
            if(this.hasFilterForField(filterDefinition, 'filter_object_type')) {
                return false;
            }else if(contextLayout == 'records' && this.filterContainsForbiddenParentTypes(filterDefinition, 'parent_type', contextModule)){
                return false;
            }else if(contextLayout == 'record' &&  (this.hasFilterForField(filterDefinition, 'parent_type') || this.hasFilterForField(filterDefinition, 'parent_id'))){
                return false;
            }
        }

        return true;
    },

    hasFilterForField: function(filterDef, targetFieldName){
        if(_.isEmpty(filterDef))
            return false;

        var hasFilter = false;
        _.each(filterDef, function(filter, fieldName){
            if(hasFilter)
                return hasFilter;
            if(_.isString(fieldName) && fieldName == targetFieldName){
                hasFilter = true;
                return hasFilter;
            }else if(_.isObject(filter) || (_.isArray(filter) && !_.isString(filter))){
                hasFilter = this.hasFilterForField(filter, targetFieldName);
                if(hasFilter)
                    return hasFilter;
            }
        }, this);

        return hasFilter;
    },

    filterContainsForbiddenParentTypes: function(filterDef, forbiddenFieldName, fieldValue){
        if(_.isEmpty(filterDef))
            return false;

        var contains = false;
        _.each(filterDef, function(filter, fieldName){
            if(contains)
                return contains;

            if(_.isString(fieldName) && fieldName == forbiddenFieldName){
                if(_.isString(filter)){
                    contains = (filter != fieldValue);
                }
                if(contains)
                    return contains;
            }else if(_.isObject(filter) || (_.isArray(filter) && !_.isString(filter))){
                contains = this.filterContainsForbiddenParentTypes(filter, forbiddenFieldName, fieldValue);
                if(contains)
                    return contains;
            }
        }, this);

        return contains;
    },

})
