<?php

use Sugarcrm\Sugarcrm\Security\HttpClient\ExternalResourceClient;

require_once 'modules/PX_NeedToDoOutfittersLicense/license/PX_NeedToDoOutfittersLicenseHandler.php';

class PX_NeedToDoOutfittersLicenseHandlerApi
{
    private function get_default_payload($module,$custom_data = array())
    {
        global $sugar_config, $sugar_flavor, $db;
        $not_set_value = 'not set';
        $data = [];

        //load license validation config
        $outfitters_config = [];
        require('modules/'.$module.'/license/config.php');

        // set the key, check custom data first in case developer wants to explicitly set a key
        if (empty($custom_data['key'])) {
            $key = PX_NeedToDoOutfittersLicenseHandler::getKey($module);
            $data['key'] = empty($key) ? false : $key;
        } else {
            $data['key'] = $custom_data['key'];
        }

        // set public key, check custom data first in case developer wants to explicitly set a public key
        if (empty($custom_data['public_key'])) {
            $data['public_key'] = empty($outfitters_config['public_key']) ? $not_set_value : $outfitters_config['public_key'];
        } else {
            $data['public_key'] = $custom_data['public_key'];
        }

        // set user counts, check custom data first in case developer wants to explicitly set a user count
        if (!empty($custom_data['user_count'])) {
            $data['user_count'] = $custom_data['user_count'];
        } else if (isset($outfitters_config['manage_licensed_users']) && $outfitters_config['manage_licensed_users'] == true) {
            //using built-in user license management
            $licensed_users = 0;
            $sql = "SELECT count(*) as the_count FROM so_users WHERE shortname = '".$db->quote($outfitters_config['shortname'])."'";
            $result = $db->query($sql,false,'Unable to reset licensed users for '.$outfitters_config['shortname']);
            $row = $db->fetchByAssoc($result);
            if (!empty($row)) {
                $licensed_users = $row['the_count'];
            }
            $data['user_count'] = $licensed_users;
        } else {
            $active_users = get_user_array(false,'Active','',false,'',' AND portal_only=0 AND is_group=0');
            $data['user_count'] = empty($active_users) ? 0 : count($active_users);
        }

        // other user count types
        $active_group_users = get_user_array(false,'Active','',false,'',' AND portal_only=0 AND is_group=1');
        $data['group_user_count'] = empty($active_group_users) ? 0 : count($active_group_users);
        $active_admin_users = get_user_array(false,'Active','',false,'',' AND portal_only=0 AND is_group=0 AND is_admin=1');
        $data['admin_user_count'] = empty($active_admin_users) ? 0 : count($active_admin_users);

        // get sugar edition from global far
        $data['sugar_edition'] = empty($sugar_flavor) ? $not_set_value : $sugar_flavor;

        // get sugar fork
        $fork = 'SugarCRM';
        $data['sugar_fork'] = isset($sugar_config['suitecrm_version']) ? 'SuiteCRM' : $fork;

        // get sugar_config data
        $data['db_type'] = empty($sugar_config['dbconfig']['db_type']) ? $not_set_value : $sugar_config['dbconfig']['db_type'];
        $data['developerMode'] = (!empty($sugar_config['developerMode']) && $sugar_config['developerMode']===true ? 'true' : 'false');
        $data['host_name'] = empty($sugar_config['host_name']) ? $not_set_value : $sugar_config['host_name'];
        $data['package_scan'] = (!empty($sugar_config['moduleInstaller']) && !empty($sugar_config['moduleInstaller']['packageScan']) && $sugar_config['moduleInstaller']['packageScan']===true ? 'true' : 'false');
        $data['sugar_version'] = empty($sugar_config['sugar_version']) ? $not_set_value : $sugar_config['sugar_version'];
        $data['site_url'] = empty($sugar_config['site_url']) ? $not_set_value : $sugar_config['site_url'];

        // attempt to get addon version
        $data['addon_version'] = $not_set_value;
        if (!empty($outfitters_config['name'])) {
            $result = $db->query("select version from upgrade_history where id_name = '".$db->quote($outfitters_config['name'])."' order by date_entered DESC");
            $row = $db->fetchByAssoc($result);
            if (!empty($row)) {
                $data['addon_version'] = empty($row['version']) ? $not_set_value : $row['version'];
            } else {
                $data['addon_version'] = "No Versions Found";
            }
        } else {
            $data['addon_version'] = "Error: 'name' not set in outfitters_config.";
        }

        // process any metadata passed in
        // must be a php array, we will json_encode it for sending
        $data['metadata'] = '';
        if (!empty($custom_data['metadata']) && is_array($custom_data['metadata'])) {
            $data['metadata'] = json_encode($custom_data['metadata']);
        }

        return $data;
    }

    public static function tag($action,$payload)
    {
        return true;
    }

    public static function call($module,$path,$custom_data=array(),$method='post')
    {
        if (empty($module)) {
            return array(
                'success' => false,
                'result' => 'Module is not defined.'
            );
        }

        if (empty($path)) {
            return array(
                'success' => false,
                'result' => 'API Call Path is not defined.'
            );
        }

        if (!is_array($custom_data)) {
            return array(
                'success' => false,
                'result' => 'Data for API Call must be an array.'
            );
        }

        //load license validation config and generate url from config
        $outfitters_config = [];
        require('modules/'.$module.'/license/config.php');
        $url = $outfitters_config['api_url'] . '/' . $path;

        // get default payload data
        $data = static::get_default_payload($module,$custom_data);

        if(empty($data['key'])) {
            return array(
                'success' => false,
                'result' => 'Key could not be found locally. Please go to the license configuration tool and enter your key.'
            );
        }

        // ExternalResourceClient POST JSON
        $client = new ExternalResourceClient(60, 10);
        try {
            if ($method === 'post') {
                $response = $client->post($url, json_encode($data), ['Content-Type' => "application/json"]);
            } else {
                $url .= '?' . http_build_query($data);
                $response = $client->get($url);
            }
            $httpCode = $response->getStatusCode();
            $result = !empty($response) ? json_decode($response->getBody()->getContents(), true) : null;
            if ($httpCode != 200) {
                $GLOBALS['log']->fatal('PX_NeedToDoOutfittersLicenseHandlerApi::call(): HTTP Request failed: '.print_r($httpCode,true));
                return array(
                    'success' => false,
                    'result' => $result
                );
            } else {
                return array(
                    'success' => true,
                    'result' => $result
                );
            }
        } catch (Exception $e) {
            $GLOBALS['log']->fatal('PX_NeedToDoOutfittersLicenseHandlerApi::call(): Unable to validate license. Please configure the firewall to allow requests to '.$outfitters_config['api_url'].'/key/validate and make sure that SSL certs are up to date on the server.');
            return array(
                'success' => false,
                'result' => 'PX_NeedToDoOutfittersLicenseHandlerApi::call(): Unable to validate the license key. Please configure the firewall to allow requests to '.$outfitters_config['api_url'].'/key/validate and make sure that SSL certs are up to date on the server.'
            );
        }
    }

}
