<?php

require_once 'modules/PX_NeedToDoViewerComments/PX_NeedToDoViewerComments_sugar.php';
require_once 'custom/PxRoutines/NeedToDoViewer/Include/utils/NeedToDoEventsLogicHook.php';

class PX_NeedToDoViewerComments extends PX_NeedToDoViewerComments_sugar
{

    public $table_name = 'px_needtodoviewercomments';
    public $object_name = 'PX_NeedToDoViewerComments';
    public $module_dir = 'PX_NeedToDoViewerComments';

    //todo: remove?
    public $nestedCommentList = [];

    public $id;
    public $name = '';
    public $date_entered;
    public $date_modified;
    public $parent_type;
    public $parent_id;
    public $data = '{}';
    public $created_by;
    public $created_by_name;

    public $last_comment = '{}';
    public $last_comment_bean;
    public $comment_count = 0;

    private $parentBean;

    private $conversationParticipants;

    public function __construct()
    {
        parent::__construct();
    }

    public function getConversationParticipants()
    {
        return $this->conversationParticipants;
    }

    /**
     * Method that returns a JSON representation of the bean.
     * @return string
     */
    public function toJson()
    {
        global $current_user;

        $this->retrieve();
        $sfh = new SugarFieldHandler();
        $data = array();
        require_once 'include/api/RestService.php';
        $service = new RestService();
        $service->user = $current_user;
        foreach ($this->field_defs as $fieldName => $properties) {
            $type = !empty($properties['custom_type']) ? $properties['custom_type'] : $properties['type'];

            $field = $sfh->getSugarField($type);
            if ($field != null && isset($this->$fieldName)) {
                $field->apiFormatField($data, $this, array(), $fieldName, $properties, array(), $service);
            }
        }
        return json_encode($data);
    }

    public function getCommentDataAsArray()
    {
        if (is_string($this->data)) {
            $data = json_decode($this->data, true);
            if (!empty($data) && is_array($data) && json_last_error() != JSON_ERROR_SYNTAX) {
                return $data;
            }
        }
        return [];
    }

    /**
     * Saves the current comment.
     * @param  boolean $check_notify
     * @return string|bool           GUID of saved comment or false.
     */
    public function save($check_notify = false)
    {
        $this->processSubjectBeforeSave();
        $this->processDataBeforeSave();

        $parent = $this->getParentBean();

        if (!empty($parent) && $parent->id) {
            $isNew = (empty($this->id) || $this->new_with_id);

            $this->beforeSaveChat();

            if (parent::save($check_notify)) {
                $parent->addComment($this, $isNew);
                if ($isNew) {
                    $this->conversationParticipants = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers\ConversationParticipants($this);
                    $this->subscribeUsersToConversation();
                    $this->notifyMentionedUsers();
                }
                return $this->id;
            }
        }
        return false;
    }

    protected function processSubjectBeforeSave()
    {
        if (!empty($this->name)) {
            $this->name = SugarCleaner::cleanHtml($this->name);
        }
    }

    protected function processDataBeforeSave()
    {
        if (is_string($this->data)) {
            $this->data = json_decode($this->data, true);
        }

        if (!empty($this->data['value'])) {
            $this->data['value'] = SugarCleaner::cleanHtml($this->data['value']);
        }

        if (!is_string($this->data)) {
            $this->data = json_encode($this->data);
        }
    }

    public function getParentBean()
    {
        if (!isset($this->parentBean)) {
            $this->parentBean = BeanFactory::getBean($this->parent_type, $this->parent_id);
        }

        return $this->parentBean;
    }

    protected function notifyMentionedUsers()
    {
        $commentNotifier = new Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers\Notification\NeedToDoViewerCommentNotifier();
        $commentNotifier->init($this);
        $commentNotifier->notify();
    }

    protected function subscribeUsersToConversation()
    {
        $subscriber = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers\NeedToDoViewerCommentsUsers($this);
        $subscriber->save();
    }

    public function retrieve($id = '-1', $encode = false, $deleted = true)
    {
        try {
            require_once 'custom/PxRoutines/NeedToDoViewer/Package/License/Control.php';
            $control = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Control();
            $control->checkLicenseForCurrentUser();
        } catch(Exception $e) {
            return null;
        }

        $encode = false;
        $result = parent::retrieve($id, $encode, $deleted);
        return $result;
    }

    /**
     * Overwrite the notifications handler.
     */
    public function _sendNotifications($check_notify)
    {
        return false;
    }

    public function get_notification_recipients()
    {
        return array();
    }


    /**
     * init reply-chat
     */
    public function initChat()
    {
        if (!isset($this->last_comment_bean)) {
            $this->last_comment_bean = BeanFactory::getBean('PX_NeedToDoViewerComments');
        }
    }

    /**
     * add reply to comment
     */
    public function addComment(PX_NeedToDoViewerComments $comment, $isNew = false)
    {
        $this->initChat();

        if ($this->id && $comment->id && $comment->parent_id == $this->id) {
            if ($isNew) {
                $this->comment_count++;
            }
            $this->last_comment_bean = $comment;
            $this->save();
            return true;
        }
        return false;
    }

    /**
     * @param $comment_id
     * Remove comment
     */
    public function deleteComment($comment_id)
    {
        $this->initChat();

        if ($comment_id && $this->id) {
            $comment = BeanFactory::getBean("PX_NeedToDoViewerComments", $comment_id);
            if ($comment->parent_id == $this->id) {
                $comment->mark_deleted($comment_id);
                $this->comment_count--;
                $this->load_relationship('comments');
                $params = array('limit' => 1, 'orderby' => 'date_entered DESC');
                $linkResult = $this->comments->query($params);
                $last_comment_id = null;
                $linkResult = array_keys($linkResult['rows']);
                if (count($linkResult)) {
                    $last_comment_id = $linkResult[0];
                }
                $this->last_comment_bean = BeanFactory::getBean('PX_NeedToDoViewerComments', $last_comment_id);
                $this->save();
            }
        }
    }

    /**
     * Before save chat
     */
    public function beforeSaveChat()
    {
        $this->initChat();

        if (!empty($this->last_comment_bean)) {
            $this->last_comment = $this->last_comment_bean->toJson();
        }
    }

    function call_custom_logic($event, $arguments = [])
    {
        if (!isset($this->processed) || $this->processed == false) {
            //add some logic to ensure we do not get into an infinite loop
            if (!empty($this->logicHookDepth[$event])) {
                if ($this->logicHookDepth[$event] > $this->max_logic_depth) {
                    return;
                }
            } else {
                $this->logicHookDepth[$event] = 0;
            }

            //we have to put the increment operator here
            //otherwise we may never increase the depth for that event in the case
            //where one event will trigger another as in the case of before_save and after_save
            //Also keeping the depth per event allow any number of hooks to be called on the bean
            //and we only will return if one event gets caught in a loop. We do not increment globally
            //for each event called.
            $this->logicHookDepth[$event]++;

            //method defined in 'include/utils/LogicHook.php'
            $logicHook = new NeedToDoEventsLogicHook();
            $logicHook->setBean($this);
            $logicHook->call_custom_logic($this->module_dir, $event, $arguments);
            $this->logicHookDepth[$event]--;
            //Fire dependency manager dependencies here for some custom logic types.
            if (in_array($event, array('after_relationship_add', 'after_relationship_delete', 'before_delete'))) {
                $this->updateRelatedCalcFields($arguments['link'] ?? "");
            }
        }
    }

}
