({
    extendsFrom: 'SubpanelListView',
    fallbackFieldTemplate: 'list',
    commentsNestingLevel: 2,
    plugins: ['ErrorDecoration', 'Editable', 'SugarLogic', 'Pagination',
        'ResizableColumns', 'MassCollection'],

    contextEvents: {
        "list:editall:fire": "toggleEdit",
        "list:editrow:fire": "editClicked",
        "list:unlinkrow:fire": "warnUnlink"
    },

    /**
     * @override
     * @param {Object} options
     */
    initialize: function(options) {
        this._super("initialize", [options]);

        this.context.set('limit', -1);
        //supanel-list extends indirectly ListView, and `limit` determines # records displayed
        this.limit = -1;

        ///NEW
        this.template = app.template.getView('comments-flex-list', 'PX_NeedToDoViewerComments');


        // this.collection.on('sync reset add ', function() {
        //     this.toggleMessageIfEmpty();
        // }, this);
    },

    _render: function() {
        this._super('_render');

        _.each(this.collection.models, function (model) {
            var view = app.view.createView({
                context: this.context,
                name: "need-to-do-event-comment",
                type: "need-to-do-event-comment",
                module: this.module,
                layout: this,
                model: model,
                inlineFetch: true,
                commentsNestingLevel: this.commentsNestingLevel - 1,
                needToDoEventLayout: this
            });
            this.$el.find('.ntdv-comments-subpanel').append(view.$el);
            view.render();
        }, this);

    },

    /**
     * When parent recordlist's initialize is invoked (above), this will get called
     * and populate our the list's meta with the proper view subpanel metadata.
     *
     * @private
     * @param {Object} options
     * @return {Object} The view metadata for this module's subpanel.
     */
    _initializeMetadata: function(options) {
        return  _.extend({},
            app.metadata.getView(null, 'subpanel-list'),
            app.metadata.getView(options.module, 'record-list'),
            app.metadata.getView(options.module, 'subpanel-list')
        );
    },

    /**
     * Unlink (removes) the selected model from the list view's collection
     */
    unlinkModel: function() {
        var self = this,
            model = this._modelToUnlink;

        model.destroy({
            //Show alerts for this request
            showAlerts: {
                'process': true,
                'success': {
                    messages: self.getUnlinkMessages(self._modelToUnlink).success
                }
            },
            relate: true,
            success: function() {
                var redirect = self._targetUrl !== self._currentUrl;
                self._modelToUnlink = null;
                self.collection.remove(model, { silent: redirect });

                if (redirect) {
                    self.unbindBeforeRouteUnlink();
                    //Replace the url hash back to the current staying page
                    app.router.navigate(self._targetUrl, {trigger: true});
                    return;
                }

                // We trigger reset after removing the model so that
                // panel-top will re-render and update the count.
                self.collection.trigger('reset');
                self.render();
            }
        });
    },

    /**
     * Pre-event handler before current router is changed
     *
     * @return {Boolean} true to continue routing, false otherwise
     */
    beforeRouteUnlink: function () {
        if (this._modelToUnlink) {
            this.warnUnlink(this._modelToUnlink);
            return false;
        }
        return true;
    },

    /**
     * Formats the messages to display in the alerts when unlinking a record.
     *
     * @param {Data.Bean} model The model concerned.
     * @return {Object} The list of messages.
     * @return {string} return.confirmation Confirmation message.
     * @return {string} return.success Success message.
     */
    getUnlinkMessages: function(model) {
        var messages = {};
        var name = Handlebars.Utils.escapeExpression(app.utils.getRecordName(model)).trim();
        var context = app.lang.getModuleName(model.module).toLowerCase() + ' ' + name;

        messages.confirmation = app.utils.formatString(app.lang.get('NTC_UNLINK_CONFIRMATION_FORMATTED'), [context]);
        messages.success = app.utils.formatString(app.lang.get('NTC_UNLINK_SUCCESS'), [context]);
        return messages;
    },

    /**
     * Popup dialog message to confirm unlink action
     *
     * @param {Backbone.Model} model the bean to unlink
     */
    warnUnlink: function(model) {
        var self = this;
        this._modelToUnlink = model;

        self._targetUrl = Backbone.history.getFragment();
        //Replace the url hash back to the current staying page
        if (self._targetUrl !== self._currentUrl) {
            app.router.navigate(this._currentUrl, {trigger: false, replace: true});
        }

        app.alert.show('unlink_confirmation', {
            level: 'confirmation',
            messages: self.getUnlinkMessages(model).confirmation,
            onConfirm: _.bind(self.unlinkModel, self),
            onCancel: function() {
                self._modelToUnlink = null;
            }
        });
    },

    /**
     * Popup browser dialog message to confirm unlink action
     *
     * @return {String} the message to be displayed in the browser alert
     */
    warnUnlinkOnRefresh: function() {
        if (this._modelToUnlink) {
            return this.getUnlinkMessages(this._modelToUnlink).confirmation;
        }
    },

    /**
     * Detach the event handlers for warning unlink
     */
    unbindBeforeRouteUnlink: function() {
        app.routing.offBefore("route", this.beforeRouteUnlink, this);
        $(window).off("beforeunload.unlink" + this.cid);
    },

    /**
     * @override
     * @private
     */
    _dispose: function() {
        this.unbindBeforeRouteUnlink();
        this._super('_dispose');
    },

    getStickyItemEl: function(){
        return null;
    },

    getCurrentParentEl: function(){
        var mainPaneScrollEl = this.$el.closest('.main-pane');
        if(!_.isEmpty(mainPaneScrollEl)) {
            return mainPaneScrollEl;
        }

        return null;
    },

    getCurrentParentTop: function(){
        var parent = this.$el[0],
            scrollTop = this.getMainPaneOffsetHeight(),
            currentParentTop = 0;

        currentParentTop = parent.offsetTop + scrollTop;

        while(parent.offsetParent) {
            parent = parent.offsetParent;
            currentParentTop += (parent.offsetTop);
        }

        return currentParentTop;
    },

    getMainPaneOffsetHeight: function(){
        var mainPaneEl = this.$el.closest('.main-pane');
        if(!_.isEmpty(mainPaneEl)){
            return mainPaneEl[0].scrollTop;
        }
        return 0;
    },

})
